@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -683,6 +683,8 @@ cdw_rv_t cdw_config_general_copy(cdw_config_general_t *dest, cdw_config_general_
 	dest->display_hidden_files        = src->display_hidden_files;     /* This one is in "general.", but is not displayed in Configuration window. */
 	dest->support_dvd_rp_dl           = src->support_dvd_rp_dl;
 	dest->show_dvd_rw_support_warning = src->show_dvd_rw_support_warning;
+	dest->fs_lynx_like_motion         = src->fs_lynx_like_motion;
+
 
 	return CDW_OK;
 }
@@ -1101,6 +1103,8 @@ cdw_rv_t cdw_config_general_set_defaults(cdw_config_general_t *general)
 	   box, and will set this flag to "false", to not to annoy
 	   user with repeated messages. */
 	general->show_dvd_rw_support_warning = true;
+	/* false: preserve old behaviour by default. */
+	general->fs_lynx_like_motion = false;
 
 	return CDW_OK;
 }
@@ -1289,6 +1293,9 @@ void cdw_config_var_write_to_file(FILE *config_file, cdw_config_t *config)
 	fprintf(config_file, "\n# Display hidden files in browsers of native file system.\n");
 	fprintf(config_file, "general.display_hidden_files=%d\n",      config->general.display_hidden_files ? 1 : 0);
 
+	fprintf(config_file, "\n# Use Lynx-like motion (left/right arrow) when navigating native file system.\n");
+	fprintf(config_file, "general.fs_lynx_like_motion=%d\n",       config->general.fs_lynx_like_motion ? 1 : 0);
+
 	return;
 }
 
@@ -1696,6 +1703,11 @@ cdw_rv_t cdw_config_var_read_from_file(FILE *config_file, cdw_config_t *config)
 			continue;
 		}
 
+		if (!strcasecmp(option.name, "general.fs_lynx_like_motion")) {
+			cdw_string_get_bool_value(option.value, &(config->general.fs_lynx_like_motion));
+			continue;
+		}
+
 	} /* while () */
 
 	cdw_config_option_free(&option);
@@ -1984,6 +1996,7 @@ void cdw_config_debug_general_print_options(cdw_config_general_t *general)
 	cdw_vdm ("INFO:        display hidden files = %s\n",     general->display_hidden_files ? "true" : "false");
 	cdw_vdm ("INFO:           support DVD+RP DL = %s\n",     general->support_dvd_rp_dl ? "true" : "false");
 	cdw_vdm ("INFO: show DVD-RW support warning = %s\n",     general->show_dvd_rw_support_warning ? "true" : "false");
+	cdw_vdm ("INFO:      Lynx-like motion in fs = %s\n",     general->fs_lynx_like_motion ? "true" : "false");
 
 	return;
 }
@@ -2010,6 +2023,15 @@ bool cdw_config_follow_symlinks(void)
 
 
 
+bool cdw_config_fs_lynx_like_motion(void)
+{
+	return global_config.general.fs_lynx_like_motion;
+}
+
+
+
+
+
 #ifdef CDW_UNIT_TEST_CODE
 
 
