@@ -72,6 +72,8 @@ typedef struct {
 	   (yet). */
 	bool display_hidden_files;           /**< Display hidden files in browsers of native file system. */
 
+	bool fs_lynx_like_motion;            /**< Recognize left and right arrow keys as native file system navigation keys (instead of horizontal scroll keys). */
+
 	/* Options not displayed in configuration window and not saved
 	   to config file */
 	bool support_dvd_rp_dl;
@@ -156,6 +158,7 @@ cdw_id_t cdw_config_volume_id_by_label(const char *label);
 
 bool cdw_config_support_dvd_rp_dl(void);
 bool cdw_config_follow_symlinks(void);
+bool cdw_config_fs_lynx_like_motion(void);
 
 
 
