@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -189,7 +189,7 @@ cdw_rv_t cdw_config_window(void)
 #define PAGE_HW_N_FIELDS         7  /* hardware */
 #define PAGE_AUDIO_N_FIELDS      3
 #define PAGE_TOOLS_N_FIELDS     15
-#define PAGE_OTHER_N_FIELDS     12  /* log and other */
+// #define PAGE_OTHER_N_FIELDS     14  /* log and other; moved to enum in header file. TODO: do the same for other four #defines. */
 #define PAGE_UDF_N_FIELDS        2
 
 
@@ -765,13 +765,17 @@ cdw_rv_t page_other_build_fields(cdw_form_t *cdw_form, void *cdw_config, int fir
 		{ CDW_WIDGET_ID_STATIC_LABEL,     6,  first_col,         width_narrow - 5,  1,  f_volume_size_l,         _("Volume size:"),                   0 },
 		{ CDW_WIDGET_ID_DROPDOWN,         6,  width_narrow - 3,  width_narrow,      1,  f_volume_size_dd,        dropdown_makers,                     0 },
 
-		/* 2TRANS: this is input field label, user can enter into the field a custom value of max. size of ISO/UDF volume; "MB" is megabytes */
-		{ CDW_WIDGET_ID_DYNAMIC_LABEL,    7,  first_col,         width_narrow,      1,  f_cust_volume_size_l,    _("Custom volume size:"),            0 },
+		/* 2TRANS: this is input field label, user can enter into the field a custom value of max. size of ISO/UDF volume; "MB" is megabytes. This is similar text as in dropdown item label. */
+		{ CDW_WIDGET_ID_DYNAMIC_LABEL,    7,  first_col,         width_narrow - 3,  1,  f_cust_volume_size_l,    _("Custom size:"),                   0 },
 		{ CDW_WIDGET_ID_SAFE_INPUT_LINE,  7,  second_col,        CDW_CVF_WIDTH,     1,  f_cust_volume_size_i,    cvf_buffer,              CDW_CVF_WIDTH },
 		{ CDW_WIDGET_ID_STATIC_LABEL,     7,  third_col,         17,                1,  f_cust_volume_size_l2,   _("MB (MB = 2^20 B)"),               0 },
 
-		{ CDW_WIDGET_ID_CHECKBOX,        11,  first_col + 1,     1,                 1,  f_follow_symlinks_cb,    (void *) NULL,  config->general.selected_follow_symlinks ? 1 : 0 },
-		{ CDW_WIDGET_ID_TEXT,            11,  first_col + 4,     width_wide - 6,    8,  f_follow_symlinks_l,     text_makers,                         4 },
+		{ CDW_WIDGET_ID_CHECKBOX,         9,  first_col + 1,     1,                 1,  f_follow_symlinks_cb,    (void *) NULL,  config->general.selected_follow_symlinks ? 1 : 0 },
+		{ CDW_WIDGET_ID_TEXT,             9,  first_col + 4,     width_wide - 6,    8,  f_follow_symlinks_l,     text_makers,                         4 },
+
+		{ CDW_WIDGET_ID_CHECKBOX,        18,  first_col + 1,     1,                 1,  f_fs_lynx_like_motion_cb,    (void *) NULL,  config->general.fs_lynx_like_motion ? 1 : 0 },
+		/* 2TRANS: this is input field label, "Lynx" is a text-mode web browser */
+		{ CDW_WIDGET_ID_STATIC_LABEL,    18,  first_col + 4,     width_wide - 6,    1,  f_fs_lynx_like_motion_l,     _("Lynx-like motion in file system"),                     0 },
 
 		/* guard */
 		{ -1,                             0,  0,                 0,                 0,  0,                       (void *) NULL,                       0 } };
@@ -903,6 +907,7 @@ void cdw_config_window_save_config_fields(cdw_config_t *config)
 	}
 
 	config->general.selected_follow_symlinks = cdw_form_get_checkbox_state(cdw_form, f_follow_symlinks_cb);
+	config->general.fs_lynx_like_motion = cdw_form_get_checkbox_state(cdw_form, f_fs_lynx_like_motion_cb);
 
 #if 0
 
@@ -959,15 +964,15 @@ int page_tools_toggle_manual_selection(cdw_form_t *cdw_form, void *state)
 int page_other_handle_enter_on_log_path_button(cdw_form_t *cdw_form, __attribute__((unused)) void *dummy)
 {
 	/* 2TRANS: this is title of window with file picker */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Log file"),
-					     /* 2TRANS: this is message in file picker
-						window, explaining what file should be
-						selected */
-					     _("Select new or existing log file"),
-					     &(local_config->general.log_fullpath),
-					     CDW_FS_FILE,
-					     R_OK | W_OK,
-					     CDW_FS_NEW | CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Log file"),
+				       /* 2TRANS: this is message in file picker
+					  window, explaining what file should be
+					  selected */
+				       _("Select new or existing log file"),
+				       &(local_config->general.log_fullpath),
+				       CDW_FS_FILE,
+				       R_OK | W_OK,
+				       CDW_FS_NEW | CDW_FS_EXISTING);
 	if (crv == CDW_OK) {
 		int rv = set_field_buffer(cdw_form->fields[f_log_fp_i], 0, local_config->general.log_fullpath);
 		if (rv != E_OK) {
@@ -990,15 +995,15 @@ int page_other_handle_enter_on_log_path_button(cdw_form_t *cdw_form, __attribute
 int page_audio_handle_enter_on_audiodir_button(cdw_form_t *cdw_form, __attribute__((unused)) void *dummy)
 {
 	/* 2TRANS: this is title of window with file picker */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Audio output dir"),
-					     /* 2TRANS: this is message in file picker
-						window, explaining what file should be
-						selected */
-					     _("Select existing output dir"),
-					     &(local_config->audiodir),
-					     CDW_FS_DIR,
-					     R_OK | W_OK | X_OK,
-					     CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Audio output dir"),
+				       /* 2TRANS: this is message in file picker
+					  window, explaining what file should be
+					  selected */
+				       _("Select existing output dir"),
+				       &(local_config->audiodir),
+				       CDW_FS_DIR,
+				       R_OK | W_OK | X_OK,
+				       CDW_FS_EXISTING);
 	if (crv == CDW_OK) {
 		int rv = set_field_buffer(cdw_form->fields[f_audiodir_i], 0, local_config->audiodir);
 		if (rv != E_OK) {
@@ -1042,8 +1047,8 @@ static cdw_id_clabel_t cdw_config_volume_size_items[CDW_CONFIG_VOLUME_ITEMS_MAX]
 	/* 2TRANS: this is dropdown item label: DVD+R DL */
 	{ CDW_CONFIG_VOLUME_SIZE_DVD_RP_DL,          gettext_noop("DVD+R DL (8.5 GB)") },
 	/* 2TRANS: this is dropdown item label: custom value of
-	   ISO volume; user can enter arbitrary natural number */
-	{ CDW_CONFIG_VOLUME_SIZE_CUSTOM,             gettext_noop("Custom value") },
+	   ISO volume; user can enter arbitrary natural number. This is similar text as in input field label. */
+	{ CDW_CONFIG_VOLUME_SIZE_CUSTOM,             gettext_noop("Custom size") },
 	/* 2TRANS: this is dropdown item label:
 	   "automatic" = automatic detection/resolution of size of ISO volume */
 	{ CDW_CONFIG_VOLUME_SIZE_AUTO,               gettext_noop("Get size from disc") }
@@ -1220,5 +1225,5 @@ int cdw_config_ui_tabs_window_form_reader(void *twindow)
 	   function pointer.
 
 	   Use form driver, and tell it to start at initial field fi. */
-	return cdw_form_driver(options_pages[id].cdw_form, fi);
+	return cdw_form_driver_new(options_pages[id].cdw_form, &fi);
 }
