@@ -25,6 +25,8 @@ enum cdw_tool {
 
 	CDW_TOOL_XORRISO,
 
+	CDW_TOOL_MKUDFFS,
+
 	CDW_TOOL_MD5SUM,
 	CDW_TOOL_SHA1SUM,
 	CDW_TOOL_SHA224SUM,
@@ -32,20 +34,33 @@ enum cdw_tool {
 	CDW_TOOL_SHA384SUM,
 	CDW_TOOL_SHA512SUM,
 
+	CDW_TOOL_TRUNCATE,
+	CDW_TOOL_SUDO,
+	CDW_TOOL_MOUNT,
+	CDW_TOOL_UMOUNT,
+	CDW_TOOL_SYNC,
+	CDW_TOOL_RSYNC,
+
+
 	CDW_TOOL_LIBCDIO
 }; /** \brief Tool used to perform given task */
 
 #define CDW_EXT_TOOLS_N_DIGEST_TOOLS 6 /* md5sum, sha1sum, sha224sum, sha256sum, sha384, sha512sum */
 
 
-/* twelve external tools available:
+
+
+/* There are 19 external tools that cdw will look for with "which":
    cdrecord, mkisofs,
-   growisofs, dvd-rw+mediainfo, dvd-rw+format, xorriso,
-   md5sum, sha1sum, sha224sum, sha256sum, sha384, sha512sum;
+   growisofs, dvd-rw+mediainfo, dvd-rw+format,
+   xorriso,
+   mkudffs,
+   md5sum, sha1sum, sha224sum, sha256sum, sha384, sha512sum,
+   truncate, sudo, mount, umount, sync, rsync;
 
    I'm not including "libcdio", because this is a library (although
-   I should check for it somehow)  */
-#define CDW_EXT_TOOLS_N_TOOLS 12
+   I should check for it somehow). */
+#define CDW_EXT_TOOLS_N_TOOLS 19
 
 
 /* maximal number of instances of a tool (locations in filesystem), that can
@@ -66,13 +81,26 @@ bool        cdw_ext_tools_is_digest_tool(cdw_id_t tool_id);
 cdw_id_t cdw_ext_tools_config_get_cd_handler_id(void);
 cdw_id_t cdw_ext_tools_config_get_dvd_handler_id(void);
 cdw_id_t cdw_ext_tools_config_get_iso9660_id(void);
+
+/* For asking about some very specific details. */
 bool cdw_ext_tools_config_is_manual(void);
 bool cdw_ext_tools_is_cdrecord_wodim(void);
-bool cdw_ext_tools_digest_tool_available(void);
+
+/* Check if specific subset of tools is available, or - to put it
+   differently - if tools detected by cdw allow performing some
+   specific task. */
+bool cdw_ext_tools_digest_tool_available(void);           /* Any tool needed to calculate digest of a file (md5, sha). */
+bool cdw_ext_tools_iso9660_sa_tool_available(void);       /* Any tool capable of creating from A to Z a stand-alone (sa) ISO9660 image file. */
+bool cdw_ext_tools_udf_sa_tools_tools_available(void);    /* A complete toolset needed to create from A to Z a stand-alone (sa) UDF image file. */
+
+
+int cdw_ext_tools_get_n_iso9660_handlers(void);
+int cdw_ext_tools_get_n_udf_handlers(void);
 
 typedef struct {
 	bool manual_selection;
 	CDW_DROPDOWN *iso9660;
+	CDW_DROPDOWN *udf; /* FIXME: there is no UDF dropdown in configuration window. I'm including this for now, but probably should be removed. */
 	CDW_DROPDOWN *cd_handlers;
 	CDW_DROPDOWN *dvd_handlers;
 	CDW_DROPDOWN *mkisofs;
@@ -86,6 +114,7 @@ bool cdw_ext_tools_set_burn_tool(cdw_disc_t *disc, cdw_id_label_t *burn_tool, cd
 bool cdw_ext_tools_set_digest_tool(cdw_id_label_t *tool);
 bool cdw_ext_tools_set_iso9660_otf_tool(cdw_disc_t *disc, cdw_id_label_t *tool);
 bool cdw_ext_tools_set_iso9660_sa_tool(cdw_id_label_t *tool);
+bool cdw_ext_tools_set_udf_sa_tool(cdw_id_label_t *tool);
 bool cdw_ext_tools_set_erase_tool(cdw_disc_t *disc, cdw_id_label_t *tool);
 bool cdw_ext_tools_set_media_info_tool(cdw_disc_t *disc, cdw_id_label_t *tool);
 
