@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -757,13 +757,13 @@ cdw_rv_t cdw_mkudffs_options_save_p2(cdw_udf_t *udf, cdw_form_t *cdw_form)
 int cdw_mkudffs_options_on_mount_point(__attribute__((unused)) void *dummy1, __attribute__((unused)) void *dummy2)
 {
 	/* 2TRANS: this is title of dialog window */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Path to mount point"),
-					     /* 2TRANS: this is message in dialog window;
-					     below it there is an input field where user can
-					     enter path to ISO9660 or UDF image file */
-					     _("Please enter FULL path to mount point (mount directory):"),
-					     &(options_window.udf->mount_point),
-					     CDW_FS_DIR, R_OK | W_OK, CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Path to mount point"),
+				       /* 2TRANS: this is message in dialog window;
+					  below it there is an input field where user can
+					  enter path to ISO9660 or UDF image file */
+				       _("Please enter FULL path to mount point (mount directory):"),
+				       &(options_window.udf->mount_point),
+				       CDW_FS_DIR, R_OK | W_OK, CDW_FS_EXISTING);
 
 	cdw_main_window_wrefresh();
 
