@@ -5,12 +5,21 @@
 
 
 
+#include <dirent.h> /* PATH_MAX */
+#include <limits.h> /* PATH_MAX on Alpine Linux */
+
+
+
+
+
 /* About 'ENTER' and 'ESCAPE':
    ncurses defines some symbolic names for the two keys, but
    they tend not to work. '10' and '27' seem to work fine. */
 #define CDW_KEY_ENTER    10
 #define CDW_KEY_ESCAPE   27
 #define CDW_KEY_TAB     '\t'
+#define CDW_KEY_BTAB    KEY_BTAB
+#define CDW_KEY_SPACE    ' '
 /* End of Transmission, Ctrl+D. Right now used in place where child
    process is waiting for keyboard input. */
 #define CDW_KEY_EOT       4
@@ -36,6 +45,10 @@ enum {
 };
 
 
+#define CDW_1_GIGA    1073741824   /* 1024 * 1024 * 1024 */
+#define CDW_1_MEGA       1048576   /* 1024 * 1024 */
+
+
 
 /**
  * \brief Return values of most of cdw functions
@@ -63,6 +76,44 @@ typedef struct {
 } cdw_id_label_t;
 
 
+/* Some widgets have table of return keys - keys on which a driver
+   returns control to caller. 20 keys seems to be reasonable limit at
+   this time. There is no guard, code handling return keys must use
+   n_return_keys variable. */
+#define CDW_WIDGET_N_RETURN_KEYS_MAX 20
+#define N_RETURN_KEYS_MAX CDW_WIDGET_N_RETURN_KEYS_MAX
+
+
+typedef struct cdw_widget_s cdw_widget_t;
+/* Common information about widgets.
+
+   Does a widget type have this variable as a field?
+
+   CDW_WIDGET_ID_STATIC_LABEL      - no
+   CDW_WIDGET_ID_CHECKBOX          - yes
+   CDW_WIDGET_ID_DROPDOWN          - yes
+   CDW_WIDGET_ID_BUTTON            - yes
+   CDW_WIDGET_ID_TEXT              - no
+   CDW_WIDGET_ID_SAFE_INPUT_LINE   - yes
+   CDW_WIDGET_ID_DYNAMIC_LABEL     - yes
+   CDW_WIDGET_ID_FILE_SELECTOR     - yes (fs browser)
+ */
+struct cdw_widget_s {
+
+	int type_id; /* STATIC_LABEL, CHECKBOX, DROPDOWN, etc. */
+
+	int return_keys[CDW_WIDGET_N_RETURN_KEYS_MAX];
+	int n_return_keys;
+
+	void (* add_return_key)(cdw_widget_t *widget, int key);
+
+	void *self; /* A widget of type 'static label', 'checkbox', etc., owning this cdw_widget_t variable. */
+};
+
+
+
+
+
 /* ugly, this belong to user interface */
 struct cdw_form_type;
 typedef struct cdw_form_type cdw_form_t;
