@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,17 +40,20 @@
 /**
    \file cdw_file.c
 
-   Data structure for storing information about files existing in file system.
+   Data structure for storing information about files existing in
+   native file system.
+
    This module provides two basic functions:
    \li cdw_file_new() - constructor
    \li cdw_file_delete() - destructor
 
-   two utility functions:
+   three utility functions:
    \li cdw_file_duplicate()
    \li cdw_file_equal() - a predicate
+   \li cdw_file_equal_by_fullpath() - a predicate
 
    and two special functions:
-   \li cdw_file_dealloc_files_from_list() - used when deallocating a list of files
+   \li cdw_file_dealloc_files_from_list() - used to deallocate a group of files from given list
    \li cdw_file_display_file() - used by list display widget
 
    List of arguments of cdw_file_new() constructor reflects its main usage.
@@ -66,7 +69,7 @@
 
 static cdw_file_t *cdw_file_new_base(void);
 static cdw_rv_t    cdw_file_new_stat(cdw_file_t *file);
-static cdw_rv_t    cdw_file_new_fullpath(cdw_file_t *file, const char *dirpath, const char *name);
+static cdw_rv_t    cdw_file_new_fullpath(cdw_file_t *file, char const * dirpath, char const * name);
 
 
 /**
@@ -90,12 +93,12 @@ static cdw_rv_t    cdw_file_new_fullpath(cdw_file_t *file, const char *dirpath,
    arguments to cdw_file_new(), instead of one: fullpath.
 
    \param dirpath - full path to parent directory in which the file is located
-   \param name - name of the file in the parent directory
+   \param name - name of the file in given directory
 
    \return pointer to new "file" variable on success
    \return NULL on failure
 */
-cdw_file_t *cdw_file_new(const char *dirpath, const char *name)
+cdw_file_t *cdw_file_new(char const * dirpath, char const * name)
 {
 	cdw_assert (dirpath, "ERROR: \"dirpath\" argument is NULL\n");
 
@@ -148,7 +151,7 @@ cdw_file_t *cdw_file_new(const char *dirpath, const char *name)
 
 
 /**
-   \brief Call lstat()and stat() for given file
+   \brief Call lstat() and stat() for given file
 
    Call lstat() and stat() for given file to check its parameters, like
    file validity, file type (dir/file/link), file size.
@@ -162,13 +165,10 @@ cdw_rv_t cdw_file_new_stat(cdw_file_t *file)
 
 	struct stat stbuf;
 	int rv = lstat(file->fullpath, &stbuf);
-	if (rv != -1) {
+	if (rv == 0) {
 		if (S_ISLNK(stbuf.st_mode)) {
-			file->is_link = true;
-			cdw_sdm ("IS a link: \"%s\"\n", file->fullpath);
-		} else {
-			file->is_link = false;
-			cdw_sdm ("IN NOT a link: \"%s\"\n", file->fullpath);
+			file->type |= CDW_FS_LINK;
+			cdw_sdm ("INFO: is a link: \"%s\"\n", file->fullpath);
 		}
 		file->invalid = false;
 	} else {
@@ -178,34 +178,35 @@ cdw_rv_t cdw_file_new_stat(cdw_file_t *file)
 		   fullpath (created for some reason), but the file itself
 		   doesn't exist */
 		/* perror("-1 (1) because"); */
-		cdw_vdm ("ERROR: lstat() returns -1 for fullpath = \"%s\"\n", file->fullpath);
+		cdw_vdm ("ERROR: lstat() returns -1 for fullpath \"%s\"\n", file->fullpath);
 	}
 
 	/* 2: stat() called for links examines targets */
 	rv = stat(file->fullpath, &stbuf);
 	int e = errno;
-	if (rv == -1) {
+	if (rv == 0) {
+		file->invalid = false;
+	} else {
 		file->invalid = true;
 		cdw_vdm ("WARNING: stat() failed for fullpath \"%s\" (calling stat() for links examines target); error = \"%s\"\n", file->fullpath, strerror(e));
 		/* TODO: check how caller of this function behaves when CDW_SYS_ERROR is returned */
 		/* return CDW_SYS_ERROR; */
-	} else {
-		file->invalid = false;
 	}
 
 	if (S_ISDIR(stbuf.st_mode)) {
+		/* TODO: shouldn't we assign some non-zero size? */
 		file->size = 0;
-		file->type = CDW_FS_DIR;
+		file->type |= CDW_FS_DIR;
 	} else if (S_ISREG(stbuf.st_mode)) {
 		file->size = stbuf.st_size;;
-		file->type = CDW_FS_FILE;
+		file->type |= CDW_FS_FILE;
 	} else if (S_ISLNK(stbuf.st_mode)) {
 		file->size = 0;
-		file->type = CDW_FS_LINK;
+		file->type |= CDW_FS_LINK;
 	} else {
 		/* char device, block device, named pipe, socket */
 		file->size = 0;
-		file->type = CDW_FS_OTHER;
+		file->type |= CDW_FS_OTHER;
 	}
 
 	return CDW_OK;
@@ -226,14 +227,14 @@ cdw_rv_t cdw_file_new_stat(cdw_file_t *file)
    is a path to a dir, because the function can't tell if given file is
    a dir or not. So resulting fullpath is not always 100% correct.
 
-   \param file - file in which a fullpath (and other fields) needs to me set
-   \param dirpath - path to directory, one of two parts of fullpath that needs to be set
-   \param name - name of file in directory specified by dirpath; one of two parts of fullpath that needs to be set
+   \param file - file in which a fullpath (and other fields) needs to be set
+   \param dirpath - path to directory, first of two components of fullpath
+   \param name - name of file in directory specified by dirpath; second of two components of fullpath
 
-   \return CDW_GEN_ERROR on problems
+   \return CDW_ERROR on errors
    \return CDW_OK on success
 */
-cdw_rv_t cdw_file_new_fullpath(cdw_file_t *file, const char *dirpath, const char *name)
+cdw_rv_t cdw_file_new_fullpath(cdw_file_t *file, char const * dirpath, char const * name)
 {
 	char *correct_dirpath = strdup(dirpath);
 	cdw_rv_t crv = cdw_fs_correct_dir_path_ending(&correct_dirpath);
@@ -278,7 +279,7 @@ cdw_rv_t cdw_file_new_fullpath(cdw_file_t *file, const char *dirpath, const char
    Function allocates memory for "file" variable and
    initializes some of its fields to some default values.
 
-   The function does not allocate space for file fullpath nor file name.
+   The function does not allocate space for ->fullpath or ->printable_fullpath.
 
    \return pointer to newly allocated varible on success
    \return NULL on failure
@@ -295,8 +296,7 @@ cdw_file_t *cdw_file_new_base(void)
 		file->name_start = -1;
 		file->size = 0;
 		file->invalid = true;
-		file->is_link = false;
-		file->type = CDW_FS_OTHER;
+		file->type = CDW_FS_NONE;
 		file->is_ref_to_parent_dir = false;
 
 		return file;
@@ -314,13 +314,17 @@ cdw_file_t *cdw_file_new_base(void)
    all fields are copied to new "file" variable, and pointer to
    this new variable is returned.
 
+   No memory/pointer is shared between original file and its duplicate.
+
+   The function doesn't check consistency or semantic validity of data
+   in \p file. It only ensures that file->fullpath is not NULL.
+
    \return pointer to new variable being copy of argument - on success
    \return NULL on failure
 */
-cdw_file_t *cdw_file_duplicate(const cdw_file_t *file)
+cdw_file_t *cdw_file_duplicate(cdw_file_t const * file)
 {
 	cdw_assert (file, "ERROR: \"file\" argument is NULL\n");
-	cdw_assert (file->fullpath + file->name_start, "ERROR: passing file with NULL name for duplication\n"); /* TODO: is this test valid? */
 	cdw_assert (file->fullpath, "ERROR: file's fullpath is NULL\n");
 
 	cdw_file_t *copy = (cdw_file_t *) malloc(sizeof(cdw_file_t));
@@ -340,6 +344,7 @@ cdw_file_t *cdw_file_duplicate(const cdw_file_t *file)
 	} else {
 		copy->printable_fullpath = (char *) NULL;
 	}
+
 	copy->name_start = file->name_start;
 
 	copy->fullpath = strdup(file->fullpath);
@@ -352,7 +357,6 @@ cdw_file_t *cdw_file_duplicate(const cdw_file_t *file)
 
 	copy->type = file->type;
 	copy->size = file->size;
-	copy->is_link = file->is_link;
 	copy->invalid = file->invalid;
 	copy->is_ref_to_parent_dir = file->is_ref_to_parent_dir;
 
@@ -364,7 +368,7 @@ cdw_file_t *cdw_file_duplicate(const cdw_file_t *file)
 
 
 /**
-   \brief Deallocate all resources referenced to by "file" variable
+   \brief Deallocate all resources referenced by "file" variable, free the "file"
 
    Function frees() all memory referenced to by fields of \p file, and frees
    \p file itself. NULL is assigned to \p file before return, so that caller
@@ -375,7 +379,7 @@ cdw_file_t *cdw_file_duplicate(const cdw_file_t *file)
 */
 cdw_rv_t cdw_file_delete(cdw_file_t **file)
 {
-	cdw_assert (file != (cdw_file_t **) NULL, "ERROR: \"file\" argument is NULL\n");
+	cdw_assert (file, "ERROR: \"file\" argument is NULL\n");
 
 	if (*file == (cdw_file_t *) NULL) {
 		/* TODO: shouldn't we issue only a warning and return CDW_OK? */
@@ -415,13 +419,13 @@ cdw_rv_t cdw_file_delete(cdw_file_t **file)
    \return true if all fields of the variables have the same values
    \return false otherwise
 */
-bool cdw_file_equal(const void *_file1, const void *_file2)
+bool cdw_file_equal(void const * _file1, void const * _file2)
 {
 	cdw_assert (_file1, "ERROR: first argument is NULL\n");
 	cdw_assert (_file2, "ERROR: second argument is NULL\n");
 
-	const cdw_file_t *file1 = (const cdw_file_t *) _file1;
-	const cdw_file_t *file2 = (const cdw_file_t *) _file2;
+	cdw_file_t const * file1 = (cdw_file_t const *) _file1;
+	cdw_file_t const * file2 = (cdw_file_t const *) _file2;
 
 	cdw_assert (file1, "ERROR: file1 is NULL\n");
 	cdw_assert (file2, "ERROR: file2 is NULL\n");
@@ -431,50 +435,34 @@ bool cdw_file_equal(const void *_file1, const void *_file2)
 	cdw_assert (file2->fullpath, "ERROR: file2->fullpath is NULL\n");
 
 	if (!file1 || !file2) {
-		cdw_vdm ("ERROR: one (or two) file was NULL\n");
+		cdw_vdm ("ERROR: one (or both) argument is NULL\n");
 		return false;
 	}
 
 	if (!file1->fullpath || !file2->fullpath) {
-		cdw_vdm ("ERROR: one (or two) fullpath was NULL\n");
+		cdw_vdm ("ERROR: one (or both) fullpath is NULL\n");
 		return false;
 	} else {
-		if (!strcmp(file1->fullpath, file2->fullpath)) {
-			;
-		} else {
-			cdw_sdm ("INFO: files have different fullpaths\n%s\n%s\n", file1->fullpath, file2->fullpath);
+		if (0 != strcmp(file1->fullpath, file2->fullpath)) {
+			cdw_sdm ("INFO: files have different fullpaths\n    \"%s\"\n    \"%s\"\n", file1->fullpath, file2->fullpath);
 			return false;
 		}
 	}
 
-
-	/* Values of file names were tested when testing fullpaths.
-	   TODO: does this test make sense? */
-	if (file1->fullpath + file1->name_start == (char *) NULL
-	    || file2->fullpath + file2->name_start == (char *) NULL) {
-
-		cdw_vdm ("ERROR: one (or two) file names was null\n");
-		return false;
-	}
-
 	if (!file1->printable_fullpath && !file2->printable_fullpath) {
 		/* NULL "printable_fullpath" strings are allowed. */
 	} else if (file1->printable_fullpath && file2->printable_fullpath) {
-		if (!strcmp(file1->printable_fullpath, file2->printable_fullpath)) {
-			;
-		} else {
+		if (0 != strcmp(file1->printable_fullpath, file2->printable_fullpath)) {
 			cdw_sdm ("INFO: files have different printable fullpaths\n");
 			return false;
 		}
 	} else {
 		cdw_sdm ("INFO: one printable fullpath is NULL, the other is non-NULL\n");
 		return false;
-
 	}
 
-
 	if (file1->name_start != file2->name_start) {
-		cdw_sdm ("INFO: file differ in name starts\n");
+		cdw_vdm ("ERROR: different file name starts: %zd != %zd\n", file1->name_start, file2->name_start);
 		return false;
 	}
 	if (file1->is_ref_to_parent_dir != file2->is_ref_to_parent_dir) {
@@ -489,10 +477,6 @@ bool cdw_file_equal(const void *_file1, const void *_file2)
 		cdw_sdm ("INFO: files differ in file sizes\n");
 		return false;
 	}
-	if (file1->is_link != file2->is_link) {
-		cdw_sdm ("INFO: files differ in \"link\" values\n");
-		return false;
-	}
 	if (file1->invalid != file2->invalid) {
 		cdw_sdm ("INFO: files differ in \"invalid\" values\n");
 		return false;
@@ -505,7 +489,34 @@ bool cdw_file_equal(const void *_file1, const void *_file2)
 
 
 /**
-   \brief Display in given window one file and its attributes
+   \brief Compare a given fullpath with fullpath of given file
+
+   The function may be used as argument to cdw_dll_append()
+
+   \return true if given fullpath and given file->fullpath are equal
+   \return false otherwise
+*/
+bool cdw_file_equal_by_fullpath(void const * _fullpath, void const * _file)
+{
+	cdw_assert (_fullpath, "ERROR: first argument is NULL\n");
+	cdw_assert (_file, "ERROR: second argument is NULL\n");
+
+	char const * fullpath = (char const *) _fullpath;
+	cdw_file_t const * file = (cdw_file_t const *) _file;
+
+	if (0 == strcmp(fullpath, file->fullpath)) {
+		return true;
+	} else {
+		return false;
+	}
+}
+
+
+
+
+
+/**
+   \brief Display in given display one file and its attributes
 
    Display in given \p row of given \p display one file. Set font/background
    of the file properly (based on value of \p isreverse), display additional
@@ -519,9 +530,12 @@ bool cdw_file_equal(const void *_file1, const void *_file2)
    \li if set to false then item is displayed with the same foreground and
    background attributes as whole file selector window
 
+   \p row is relative to beginning of display's window. If a window
+   has only 15 rows, \p row is always in range 0-14.
+
    \param display - display to be used to show the file
-   \param row - number of row, in which to display file
-   \param data - directory entry (file or dir) item with file properties (name, type, size)
+   \param row - number of window row (zero-based), in which to display file
+   \param data - cdw_file_t entry (file or dir) to be displayed
    \param isreverse - controls attributes of displayed item
 */
 void cdw_file_display_file(void *display, void *data, size_t row, size_t h_offset, bool isreverse)
@@ -535,43 +549,36 @@ void cdw_file_display_file(void *display, void *data, size_t row, size_t h_offse
 	cdw_assert (n_cols > 0, "ERROR: we have subwindow with 0 columns, or NULL subwindow\n");
 	cdw_assert (n_rows > (int) row, "ERROR: row number larger than number of rows in subwindow\n");
 	cdw_assert (data, "ERROR: you try to display NULL file\n");
+
 	cdw_file_t *file = (cdw_file_t *) data;
+	cdw_assert (file->fullpath, "ERROR: file has no fullpath\n");
 
 	/* this fills space between item name on the left side of the
 	   window and item attributes on the right side of the window */
 	mvwhline(displ->subwindow, (int) row, 0, ' ', n_cols);
 
-	char *string = (char *) NULL;
+	char *string = file->printable_fullpath
+		? file->printable_fullpath : file->fullpath;
+
 	if (displ->display_format == CDW_LIST_DISPLAY_FORMAT_SHORT) {
-		if (file->printable_fullpath) {
-			string = file->printable_fullpath + file->name_start;
-		} else {
-			string = file->fullpath + file->name_start;
-		}
+		/* Display just a file/dir name, no path. */
+		string += file->name_start;
+
 	} else if (displ->display_format == CDW_LIST_DISPLAY_FORMAT_LONG) {
-		cdw_assert (file->fullpath, "ERROR: using file without fullpath\n");
-		if (file->printable_fullpath) {
-			string = file->printable_fullpath;
-		} else {
-			string = file->fullpath;
-		}
+		/* string = string - display full path. */
+
 	} else {
 		cdw_assert (0, "ERROR: no code to handle display format %d\n", displ->display_format);
 	}
 
-	long unsigned int attr = 0;
-	if (file->type == CDW_FS_DIR) {
-		if (isreverse) {
-			attr = A_BOLD | A_REVERSE | COLOR_PAIR(displ->colors);
-		} else {
-			attr = A_BOLD | A_NORMAL | COLOR_PAIR(displ->colors);
-		}
+	long unsigned int attr = COLOR_PAIR(displ->colors);
+	if (file->type & CDW_FS_DIR) {
+		attr |= A_BOLD;
+	}
+	if (isreverse) {
+		attr |= A_REVERSE;
 	} else {
-		if (isreverse) {
-			attr = A_REVERSE | COLOR_PAIR(displ->colors);
-		} else {
-			attr = A_NORMAL | COLOR_PAIR(displ->colors);
-		}
+		attr |= A_NORMAL;
 	}
 	(void) wattrset(displ->subwindow, attr);
 
@@ -579,7 +586,7 @@ void cdw_file_display_file(void *display, void *data, size_t row, size_t h_offse
 	if (file->invalid) {
 		strcpy(format, "!%.*s");
 	} else {
-		if (file->is_link) {
+		if (file->type & CDW_FS_LINK) {
 			strcpy(format, "~%.*s");
 		} else {
 			strcpy(format, "%.*s");
@@ -615,18 +622,18 @@ void cdw_file_display_file(void *display, void *data, size_t row, size_t h_offse
 		   item; please keep as short as possible */
 		mvwprintw(displ->subwindow, (int) row, n_cols - 5, _("E"));
 	} else {
-		if (file->type == CDW_FS_DIR) {
+		if (file->type & CDW_FS_DIR) {
 			mvwprintw(displ->subwindow, (int) row, n_cols - 5,
 				  /* 2TRANS: this is label indicating that labeled item
 				     is a directory; please keep as short as possible */
 				  _("DIR"));
 
-		} else if (file->type == CDW_FS_FILE) {
-			float size = (float) (file->size >= 1048576 ? /* 1024 * 1024 */
-					      (float) file->size / 1048576.0 :
+		} else if (file->type & CDW_FS_FILE) {
+			float size = (float) (file->size >= CDW_1_MEGA ?
+					      (float) file->size / (1.0 * CDW_1_MEGA) :
 					      (float) file->size / 1024.0);
 			mvwprintw(displ->subwindow, (int) row, n_cols - 8, "%6.1f%s", size,
-				  (file->size >= 1048576 ?
+				  (file->size >= CDW_1_MEGA ?
 				   /* 2TRANS: suffix for megabytes,
 				      please keep as short as possible */
 				   _("M") :
@@ -648,43 +655,83 @@ void cdw_file_display_file(void *display, void *data, size_t row, size_t h_offse
 
 
 
-
-
 /**
    \brief Deallocate all files that are on given list
 
-   Function frees all memory associated with all 'file' variables stored on
-   given list. The list itself is not affected, only payload of
-   the list is free()d.
+   Function frees all memory associated with all 'file' variables
+   (including the "file" variables themselves) stored on given
+   list. The list itself is not affected, only payload of the list is
+   free()d.
 
    \param list - list of files, from which you want to dealloc all files
 
-   \return CDW_OK
 */
-cdw_rv_t cdw_file_dealloc_files_from_list(cdw_dll_item_t *list)
+void cdw_file_dealloc_files_from_list(cdw_dll_item_t * list)
 {
-	if (list) {
-		for (cdw_dll_item_t *item = list; item; item = item->next) {
-			cdw_file_delete((cdw_file_t **) &(item->data));
-		}
-	} else {
+	if (!list) {
 		cdw_vdm ("WARNING: passed NULL list to the function\n");
+		return;
 	}
 
-	return CDW_OK;
+	for (cdw_dll_item_t * item = list; item; item = item->next) {
+		cdw_file_delete((cdw_file_t **) &(item->data));
+	}
+
+	return;
 }
 
 
 
 
 
+/**
+   \brief Remove last part of a path
+
+   Create a new fullpath by removing last part (file name or directory
+   name) from \p file's fullpath.
+
+   In contrast to all other places, dirpath returned by this function
+   does not end with a slash. This is a special case needed in special
+   place.
+
+   Returned pointer is owned by caller.
+
+   \param file - file with initial value of path
+
+   \return new, shortened path on success
+   \return NULL on failure
+*/
+char *cdw_file_shorten_fullpath(cdw_file_t const * file)
+{
+	ssize_t pos = cdw_fs_get_filename_start(file->fullpath);
+	if (pos == -1) {
+		return (char *) NULL;
+	}
+
+
+	/* This removes trailing slash from directory name.  The name
+	   of a directory shall not contain an ending slash.
+
+	   'pos > 1' condition is met for all directories except for
+	   root directory. */
+	if (pos > 1 && file->fullpath[pos - 1] == '/') {
+		pos--;
+	}
+
+	char *new_fullpath = strndup(file->fullpath, (size_t) pos);
+	cdw_vdm ("INFO: original fullpath  = \"%s\"\n", file->fullpath);
+	cdw_vdm ("INFO: shortened fullpath = \"%s\"\n", new_fullpath);
+	return new_fullpath;
+}
+
+
+
+
 
 #ifdef CDW_UNIT_TEST_CODE
 
 
-/* *********************** */
-/* *** unit tests code *** */
-/* *********************** */
+
 
 
 static void     test_cdw_file_equal(void);
@@ -695,6 +742,9 @@ static cdw_rv_t test_cdw_file_new_helper(char **cwd, char **dirpath, char **file
 static void     test_cdw_file_new_fullpath(void);
 
 
+
+
+
 void cdw_file_run_tests(void)
 {
 	fprintf(stderr, "testing cdw_file.c\n");
@@ -721,14 +771,13 @@ void test_cdw_file_new_base(void)
 	cdw_file_t *file = cdw_file_new_base();
 
 	cdw_assert_test (file, "ERROR: function cdw_file_new_base() failed to allocate file\n");
-	cdw_assert_test (!file->fullpath, "ERROR: fullpath in new file base is not NULL\n");
-	cdw_assert_test (file->name_start == -1, "ERROR: name start is -1 by default\n");
+	cdw_assert_test (!file->fullpath, "ERROR: ->fullpath not set to NULL\n");
+	cdw_assert_test (file->name_start == -1, "ERROR: ->name_start not set to -1\n");
 
-	cdw_assert_test (file->size == 0, "ERROR: size of new file is not 0 by default\n");
-	cdw_assert_test (file->invalid == true, "ERROR: file is not invalid by default\n");
-	cdw_assert_test (file->is_link == false, "ERROR: file is link by default\n");
-	cdw_assert_test (file->type == CDW_FS_OTHER, "ERROR: file is not CDW_FS_OTHER by default\n");
-	cdw_assert_test (file->is_ref_to_parent_dir == false, "ERROR: file is not ref to parent by default\n");
+	cdw_assert_test (file->size == 0, "ERROR: ->size not set to zero\n");
+	cdw_assert_test (file->invalid == true, "ERROR: ->invalid not set to true\n");
+	cdw_assert_test (file->type == CDW_FS_NONE, "ERROR: ->type not set to CDW_FS_NONE\n");
+	cdw_assert_test (file->is_ref_to_parent_dir == false, "ERROR: ->is_ref_to_parent_dir not set to false\n");
 
 	cdw_file_delete(&file);
 
@@ -748,6 +797,8 @@ void test_cdw_file_equal(void)
 	bool test;
 	cdw_rv_t crv;
 
+
+	/* Prepare. */
 	file1.fullpath = strdup("/home/acerion/");
 	file2.fullpath = strdup("/home/acerion/");
 	file1.printable_fullpath = (char *) NULL;
@@ -758,15 +809,16 @@ void test_cdw_file_equal(void)
 	file2.type = CDW_FS_FILE;
 	file1.size = 123456;
 	file2.size = 123456;
-	file1.is_link = true;
-	file2.is_link = true;
 	file1.invalid = true;
 	file2.invalid = true;
 	file1.is_ref_to_parent_dir = true;
 	file2.is_ref_to_parent_dir = true;
 
+
+	/* Test. */
+
 	/* at the beginning the files are equal */
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: equal files recognized as not equal\n");
 
 
@@ -774,148 +826,133 @@ void test_cdw_file_equal(void)
 	crv = cdw_string_set(&(file1.fullpath), "/my/test/path/1/");
 	cdw_assert_test (crv == CDW_OK, "ERROR: can't set new file fullpath\n");
 
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different fullpaths are recognized as equal\n");
 
 	crv = cdw_string_set(&(file1.fullpath), "/home/acerion/");
 	cdw_assert_test (crv == CDW_OK, "ERROR: can't restore old file fullpath\n");
 
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: test failed after restoring original fullpath in file1\n");
 
 
 	/* testing with different "printable_fullpath" field values */
 	crv = cdw_string_set(&(file1.printable_fullpath), "my test string");
 	cdw_assert_test (crv == CDW_OK, "ERROR: can't set file1 printable fullpath\n");
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different printable fullpaths are recognized as equal (1)\n");
 
 	crv = cdw_string_set(&(file2.printable_fullpath), "let there be chars");
 	cdw_assert_test (crv == CDW_OK, "ERROR: can't set file2 printable fullpath (1)\n");
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different printable fullpaths are recognized as equal (2)\n");
 
 	crv = cdw_string_set(&(file2.printable_fullpath), "my test string");
 	cdw_assert_test (crv == CDW_OK, "ERROR: can't set file2 printable fullpath (2)\n");
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with equal printable fullpaths are recognized as non-equal\n");
 
 
 	/* testing with different "type" field values */
 	file1.type = CDW_FS_DIR;
 	file2.type = CDW_FS_FILE;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different types aren't recognized as different (1)\n");
 
+	file1.type = CDW_FS_FILE | CDW_FS_LINK;   /* A link to a file. */
+	file2.type = CDW_FS_FILE;
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
+	cdw_assert_test (test == false, "ERROR: files with different types aren't recognized as different (2)\n");
+
 	file1.type = CDW_FS_DIR;
 	file2.type = CDW_FS_DIR;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with the same types aren't recognized as equal (1)\n");
 
 	file1.type = CDW_FS_LINK;
 	file2.type = CDW_FS_LINK;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with the same types aren't recognized as equal (2)\n");
 
 	file1.type = CDW_FS_LINK;
 	file2.type = CDW_FS_DIR;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different types aren't recognized as different (2)\n");
 
 	file1.type = CDW_FS_FILE;
 	file2.type = CDW_FS_FILE;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with restored the same types aren't recognized as the same\n");
 
 
 	/* testing with different "size" field values */
 	file1.size = 123456;
 	file2.size = 123456;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with the same sizes are recognized as different (1)\n");
 
 	file1.size = 0;
 	file2.size = 0;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with the same sizes are recognized as different (2)\n");
 
 	file1.size = 654321;
 	file2.size = 123456;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different sizes are recognized as the same (1)\n");
 
 	file1.size = 0;
 	file2.size = 123456;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different sizes are recognized as the same (2)\n");
 
 	file1.size = 123456;
 	file2.size = 123456;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with restored the same sizes are recognized as different\n");
 
 
-	/* testing with different "link" field values */
-	file1.is_link = true;
-	file2.is_link = true;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
-	cdw_assert_test (test == true, "ERROR: files with the same 'link' fields are recognized as different\n");
-
-	file1.is_link = false;
-	file2.is_link = true;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
-	cdw_assert_test (test == false, "ERROR: files with different 'link' fields are recognized as the same\n");
-
-	file1.is_link = false;
-	file2.is_link = false;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
-	cdw_assert_test (test == true, "ERROR: files with restored the same 'link' fields are recognized as different\n");
-
-
 	/* testing with different "invalid" field values */
 	file1.invalid = true;
 	file2.invalid = true;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with the same 'invalid' fields are recognized as different\n");
 
 	file1.invalid = false;
 	file2.invalid = true;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different 'invalid' fields are recognized as the same\n");
 
 	file1.invalid = false;
 	file2.invalid = false;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with restored the same 'invalid' fields are recognized as different\n");
 
 
 	/* testing with different "is_ref_to_parent_dir" field values */
 	file1.is_ref_to_parent_dir = true;
 	file2.is_ref_to_parent_dir = true;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with the same 'is_ref_to_parent_dir' fields are recognized as different\n");
 
 	file1.is_ref_to_parent_dir = false;
 	file2.is_ref_to_parent_dir = true;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == false, "ERROR: files with different 'is_ref_to_parent_dir' fields are recognized as the same\n");
 
 	file1.is_ref_to_parent_dir = false;
 	file2.is_ref_to_parent_dir = false;
-	test = cdw_file_equal((const void *) &file1, (const void *) &file2);
+	test = cdw_file_equal((void const *) &file1, (void const *) &file2);
 	cdw_assert_test (test == true, "ERROR: files with restored the same 'is_ref_to_parent_dir' fields are recognized as different\n");
 
 
-	/* end, clean up */
-	free(file1.fullpath);
-	file1.fullpath = (char *) NULL;
-	free(file2.fullpath);
-	file2.fullpath = (char *) NULL;
+	/* Clean up. */
+	CDW_STRING_DELETE(file1.fullpath);
+	CDW_STRING_DELETE(file2.fullpath);
+
+	CDW_STRING_DELETE(file1.printable_fullpath);
+	CDW_STRING_DELETE(file2.printable_fullpath);
 
-	free(file1.printable_fullpath);
-	file1.printable_fullpath = (char *) NULL;
-	free(file2.printable_fullpath);
-	file2.printable_fullpath = (char *) NULL;
 
 	fprintf(stderr, "OK\n");
 	return;
@@ -932,10 +969,10 @@ cdw_rv_t test_cdw_file_new_helper(char **cwd, char **dirpath, char **filename)
 	cdw_assert_test (!(*filename), "ERROR: filename should be pointer to NULL");
 
 	/* first get some existing dir path */
-	*cwd = (char *) malloc(PATH_MAX); /* PATH_MAX alreay includes ending null */
+	*cwd = (char *) malloc(PATH_MAX); /* PATH_MAX already includes ending NULL */
 	cdw_assert_test (*cwd, "ERROR: failed to allocate memory for cwd\n");
 
-	char *res = getcwd(*cwd, PATH_MAX - 1);
+	char *res = getcwd(*cwd, PATH_MAX); /* Returns up to N chars (including terminating NULL) via *cwd. */
 	cdw_assert_test (res, "ERROR: failed to get current working directory (1)\n");
 	cdw_assert_test (res == *cwd, "ERROR: failed to get current working directory (2)\n");
 	cdw_rv_t crv = cdw_fs_correct_dir_path_ending(cwd);
@@ -957,19 +994,20 @@ cdw_rv_t test_cdw_file_new_helper(char **cwd, char **dirpath, char **filename)
 
 
 
+
 void test_cdw_file_new_file_delete(void)
 {
 	fprintf(stderr, "\ttesting cdw_file_new() and cdw_file_delete()... ");
 
-	/* prepare arguments for cdw_file_new() */
 
+	/* Prepare. */
 	char *cwd = (char *) NULL;
 	char *dirpath = (char *) NULL;
 	char *filename = (char *) NULL;
 	test_cdw_file_new_helper(&cwd, &dirpath, &filename);
 
-	/* create file and test it */
 
+	/* Test.  */
 	cdw_file_t *file = cdw_file_new(dirpath, filename);
 	cdw_assert_test (file, "ERROR: function cdw_file_new_base() failed to allocate file\n");
 
@@ -978,26 +1016,19 @@ void test_cdw_file_new_file_delete(void)
 	cdw_assert_test (!strcmp(file->fullpath + file->name_start, filename), "ERROR: incorrect name: should be \"%s\", is: \"%s\"\n", filename, file->fullpath + file->name_start);
 	cdw_assert_test (file->size == 0, "ERROR: size of new file is not 0 for dir\n");
 	cdw_assert_test (file->invalid == false, "ERROR: existing file is invalid\n");
-	cdw_assert_test (file->is_link == false, "ERROR: dir is link\n");
 	cdw_assert_test (file->type == CDW_FS_DIR, "ERROR: dir is not recognized as dir\n");
 	cdw_assert_test (file->is_ref_to_parent_dir == false, "ERROR: current dir should not be reference to parent dir\n");
 
 	/* now test cdw_file_delete(), but there isn't much to test */
 
-	// char **f = &(file->fullpath);
 	cdw_file_delete(&file);
 	cdw_assert_test (!file, "ERROR: failed to correctly delete file, file is not NULL\n");
-	/* on some occasions this assertion fails, but valgrind doesn't
-	   complain about leaking memory related to file->fullpath */
-	// cdw_assert_test (*f == (char *) NULL, "ERROR: failed to correctly nullify file->fullpath, it doesn't point to NULL\n");
 
 
-	free(cwd);
-	cwd = (char *) NULL;
-	free(filename);
-	filename = (char *) NULL;
-	free(dirpath);
-	dirpath = (char *) NULL;
+	/* Clean up. */
+	CDW_STRING_DELETE(cwd);
+	CDW_STRING_DELETE(filename);
+	CDW_STRING_DELETE(dirpath);
 
 	fprintf(stderr, "OK\n");
 	return;
@@ -1011,15 +1042,16 @@ void test_cdw_file_duplicate(void)
 {
 	fprintf(stderr, "\ttesting cdw_file_duplicate()... ");
 
-	/* prepare arguments for cdw_file_new() */
 
+	/* Prepare. */
 	char *cwd = (char *) NULL;
 	char *dirpath = (char *) NULL;
 	char *filename = (char *) NULL;
 	test_cdw_file_new_helper(&cwd, &dirpath, &filename);
 
-	/* create file and test it (we tested creating file before, but just in case... */
 
+	/* Test. */
+	/* We tested creating file before, but just in case... */
 	cdw_file_t *file = cdw_file_new(dirpath, filename);
 	cdw_assert_test (file, "ERROR: function cdw_file_new_base() failed to allocate file\n");
 
@@ -1028,13 +1060,13 @@ void test_cdw_file_duplicate(void)
 	cdw_assert_test (!strcmp(file->fullpath + file->name_start, filename), "ERROR: incorrect name: should be \"%s\", is: \"%s\"\n", filename, file->fullpath + file->name_start);
 	cdw_assert_test (file->size == 0, "ERROR: size of new file is not 0 for dir\n");
 	cdw_assert_test (file->invalid == false, "ERROR: existing file is invalid\n");
-	cdw_assert_test (file->is_link == false, "ERROR: dir is link\n");
 	cdw_assert_test (file->type == CDW_FS_DIR, "ERROR: dir is not recognized as dir\n");
 	cdw_assert_test (file->is_ref_to_parent_dir == false, "ERROR: current dir should not be reference to parent dir\n");
 
-	/* here is a small trick to test duplicating file with non-zero size;
-	   ('file' is a directory in this testcase, and 'size' is zero;
-	   I don't want to test copying zero value) */
+	/* Small manual change to improve testing of duplicating
+	   file. I don't want to test duplicating file with non-zero
+	   size, this would be not too interesting ('file' is a
+	   directory in this testcase, and 'size' would be zero). */
 	file->size = 17345;
 
 	/* now do the real testing */
@@ -1053,26 +1085,23 @@ void test_cdw_file_duplicate(void)
 	cdw_assert_test (file->invalid == duplicate->invalid,
 			 "ERROR: file and its duplicate have different \"invalid\" field values:\nfile->invalid      = %s\nduplicate->invalid = %s\n",
 			 file->invalid ? "true" : "false", duplicate->invalid ? "true" : "false");
-	cdw_assert_test (file->is_link == duplicate->is_link,
-			 "ERROR: file and its duplicate have different \"link\" field values:\nfile->is_link      = %s\nduplicate->is_link = %s\n",
-			 file->is_link ? "true" : "false", duplicate->is_link ? "true" : "false");
 	cdw_assert_test (file->type == duplicate->type,
-			 "ERROR: file and its duplicate have different \"type\" field values:\nfile->is_link      = %d\nduplicate->is_link = %d\n",
+			 "ERROR: file and its duplicate have different \"type\" field values:\nfile->type      = %d\nduplicate->type = %d\n",
 			 file->type, duplicate->type);
 	cdw_assert_test (file->is_ref_to_parent_dir == duplicate->is_ref_to_parent_dir,
 			 "ERROR: file and its duplicate have different \"is reference to parent dir\" field values\n");
 
+
+	/* Clean up. */
 	cdw_file_delete(&file);
 	cdw_assert_test (!file, "ERROR: failed to correctly delete file, file is not NULL\n");
 	cdw_file_delete(&duplicate);
 	cdw_assert_test (!duplicate, "ERROR: failed to correctly delete file duplicate, it is not NULL\n");
 
-	free(cwd);
-	cwd = (char *) NULL;
-	free(filename);
-	filename = (char *) NULL;
-	free(dirpath);
-	dirpath = (char *) NULL;
+	CDW_STRING_DELETE(cwd);
+	CDW_STRING_DELETE(filename);
+	CDW_STRING_DELETE(dirpath);
+
 
 	fprintf(stderr, "OK\n");
 	return;
@@ -1086,7 +1115,9 @@ void test_cdw_file_new_fullpath(void)
 {
 	fprintf(stderr, "\ttesting cdw_file_new_fullpath()... ");
 
-	/* preparation, set only fields that will be needed to test function */
+
+	/* Prepare. */
+	/* Set only fields that will be needed to test function */
 	cdw_file_t file;
 	file.printable_fullpath = (char *) NULL;
 	file.name_start = -1;
@@ -1099,7 +1130,7 @@ void test_cdw_file_new_fullpath(void)
 	char *name = strdup("name");
 
 
-	/* test */
+	/* Test. */
 	cdw_rv_t crv = cdw_file_new_fullpath(&file, dirpath, name);
 	cdw_assert_test (crv == CDW_OK, "ERROR: call to cdw_file_new_fullpath() failed\n");
 	cdw_assert_test (!strcmp(file.printable_fullpath, "/this/is/a/t?st/name"),
@@ -1109,21 +1140,19 @@ void test_cdw_file_new_fullpath(void)
 			 "ERROR: incorrect value of name_start: %zd\n", file.name_start);
 
 
-	/* clean up */
-	free(dirpath);
-	dirpath = (char *) NULL;
-	free(name);
-	name = (char *) NULL;
-	free(file.fullpath);
-	file.fullpath = (char *) NULL;
-	if (file.printable_fullpath) {
-		free(file.printable_fullpath);
-		file.printable_fullpath = (char *) NULL;
-	}
+	/* Clean up.*/
+	CDW_STRING_DELETE(dirpath);
+	CDW_STRING_DELETE(name);
+	CDW_STRING_DELETE(file.fullpath);
+	CDW_STRING_DELETE(file.printable_fullpath);
+
 
 	fprintf(stderr, "OK\n");
 	return;
 }
 
 
+
+
+
 #endif /* CDW_UNIT_TEST_CODE */
