@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,8 +91,8 @@ static char *graftpoints_fullpath = (char *) NULL;
 */
 cdw_rv_t cdw_graftpoints_create_file(void)
 {
-	cdw_dll_item_t *f = cdw_file_manager_get_list_of_selected_files(); /* head */
-	cdw_assert (f != (cdw_dll_item_t *) NULL, "ERROR: called the function when list of selected files is empty\n")
+	cdw_dll_item_t *f = cdw_selected_files_get_list(); /* head */
+	cdw_assert (f, "ERROR: called the function when list of selected files is empty\n")
 
 	cdw_rv_t crv = cdw_graftpoints_open_file();
 	if (crv != CDW_OK) {
