@@ -7,6 +7,7 @@
 #include <cdio/cd_types.h>
 #include <cdio/iso9660.h>
 
+#include "config_cdw_undefine.h"
 #include "main.h"
 
 #define CDW_OFS_VOLUME_ID_LEN_MAX ISO_MAX_VOLUME_ID
@@ -48,6 +49,8 @@ const char *cdw_ofs_long_label(cdio_fs_t fs);
 const char *cdw_ofs_short_label(cdio_fs_t fs);
 bool        cdw_ofs_is_iso(cdio_fs_t fs);
 
+bool cdw_ofs_is_iso9660_file(const char *fullpath);
+
 /* debug functions */
 void cdw_ofs_print_debug(cdw_ofs_t *ofs);
 
