@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -393,12 +393,12 @@ bool cdw_burn_disc_get_source(cdw_task_t *task)
 
 	if (task->id == CDW_TASK_BURN_FROM_IMAGE) {
 		/* 2TRANS: this is the title of dialog window */
-		cdw_rv_t crv = cdw_fs_ui_file_picker(_("Path to iso image"),
-						     /* 2TRANS: this is the message in the dialog window;
-						     "writing" means writing to optical disc */
-						     _("Please enter FULL path to an existing iso image file for writing:"),
-						     &(task->image_file_fullpath),
-						     CDW_FS_FILE, R_OK, CDW_FS_EXISTING);
+		cdw_rv_t crv = cdw_file_picker(_("Path to iso image"),
+					       /* 2TRANS: this is the message in the dialog window;
+						  "writing" means writing to optical disc */
+					       _("Please enter FULL path to an existing ISO image file for writing:"),
+					       &(task->image_file_fullpath),
+					       CDW_FS_FILE, R_OK, CDW_FS_EXISTING);
 
 		cdw_main_window_wrefresh();
 		if (crv == CDW_OK) {
@@ -408,7 +408,17 @@ bool cdw_burn_disc_get_source(cdw_task_t *task)
 			source_available = true;
 		}
 	} else if (task->id == CDW_TASK_BURN_FROM_FILES) {
+
+		/* 2TRANS: this is title of dialog window */
+		cdw_processwin_create(_("Writing to disc"), "", false);
+		
+		/* 2TRANS: this is message in dialog window - cdw is putting file names of selected files on a list. */
+		cdw_processwin_display_sub_info(_("Preparing list of files..."));
+
 		cdw_rv_t crv = cdw_file_manager_create_graftpoints_file();
+
+		cdw_processwin_destroy("", false);
+
 		if (crv == CDW_OK) {
 			long long size = cdw_selected_files_get_size();
 			task->burn.data_size_mb = (((double) size) / 1024.0) / 1024.0;
