@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include <inttypes.h>
 #include <stdlib.h>
+#include <unistd.h> /* sleep() and usleep() on Alpine Linux */
 
 #include "cdw_file_manager.h"
 #include "gettext.h"
@@ -80,7 +81,7 @@ static cdw_rv_t cdw_create_image_udf(cdw_task_t *task);
 */
 cdw_rv_t cdw_create_image(void)
 {
-	if (!cdw_file_manager_number_of_selected_files()) {
+	if (!cdw_selected_files_get_number()) {
 		/* 2TRANS: this is title of dialog window */
 		cdw_buttons_dialog(_("No files selected"),
 				   /* 2TRANS: this is message in dialog window, user
@@ -128,16 +129,35 @@ cdw_rv_t cdw_create_image(void)
 		; /* Pass, CDW_IMAGE_FORMAT_ISO9660 || CDW_IMAGE_FORMAT_UDF. */
 	}
 
+	/* 2TRANS: this is title of dialog window */
+	cdw_processwin_create(_("Create image"),
+			      task->id == CDW_TASK_CREATE_IMAGE_ISO9660 ?
+			      /* 2TRANS: this is message in dialog
+				 window - creating ISO9660 image is in
+				 progress */
+			      _("Creating ISO9660 image...") :
+			      /* 2TRANS: this is message in dialog
+				 window - creating UDF image is in
+				 progress */
+			      _("Creating UDF image..."),
+			      false);
+
 
 	if (rv == CDW_IMAGE_FORMAT_ISO9660) {
 		task->id = CDW_TASK_CREATE_IMAGE_ISO9660;
 		/* create graftpoints file for mkisofs; first checks if there
 		   are any files that will be used as source data */
+
+		/* 2TRANS: this is message in dialog window - cdw is putting file names of selected files on a list. */
+		cdw_processwin_display_sub_info(_("Preparing list of files..."));
+
 		cdw_rv_t crv = cdw_file_manager_create_graftpoints_file();
 		if (crv != CDW_OK) {
 			cdw_task_delete(&task);
+			cdw_processwin_destroy(_("Error"), true);
 			return CDW_NO;
 		}
+		cdw_processwin_display_sub_info(""); /* Graftpoins file has been created, clear message about creating list of files. */
 	} else {
 		task->id = CDW_TASK_CREATE_IMAGE_UDF;
 	}
@@ -149,6 +169,7 @@ cdw_rv_t cdw_create_image(void)
 	if (crv != CDW_OK) {
 		cdw_vdm ("ERROR: failed to select tools for task\n");
 		cdw_task_delete(&task);
+		cdw_processwin_destroy(_("Error"), true);
 		return CDW_NO;
 	}
 
@@ -162,26 +183,15 @@ cdw_rv_t cdw_create_image(void)
 	}
 #endif
 
-	/* 2TRANS: this is title of dialog window */
-	cdw_processwin_create(_("Create image"),
-			      task->id == CDW_TASK_CREATE_IMAGE_ISO9660 ?
-			      /* 2TRANS: this is message in dialog
-				 window - creating ISO9660 image is in
-				 progress */
-			      _("Creating ISO9660 image...") :
-			      /* 2TRANS: this is message in dialog
-				 window - creating UDF image is in
-				 progress */
-			      _("Creating UDF image..."),
-			      true);
-
 	cdw_logging_write_separator();
 
 	/* go! */
 	if (task->create_image.tool.id == CDW_TOOL_MKISOFS) {
 		cdw_vdm ("INFO: path at call to run task: %s\n", task->image_file_fullpath);
+		cdw_processwin_add_progress_bar();
 		crv = cdw_mkisofs_run_task(task, (cdw_disc_t *) NULL);
 	} else if (task->create_image.tool.id == CDW_TOOL_XORRISO) {
+		cdw_processwin_add_progress_bar();
 		crv = cdw_xorriso_run_task(task, (cdw_disc_t *) NULL);
 	} else if (task->create_image.tool.id == CDW_TOOL_MKUDFFS) {
 		crv = cdw_create_image_udf(task);
@@ -373,7 +383,7 @@ cdw_rv_t cdw_create_image_udf(cdw_task_t *task)
 
 
 	/* No progress bar while calling truncate, mkudffs and mount. */
-	cdw_processwin_delete_progress_bar();
+	//cdw_processwin_delete_progress_bar();
 
 	cdw_logging_write("\n\nCreating UDF file: 1/6: attempting to create initial empty file:");
 	cdw_processwin_display_main_info("Creating initial .udf file");
