@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -214,12 +214,12 @@ cdw_rv_t cdw_read_disc_read_audio_disc_prepare(char **audio_file_name)
 {
 	/* 2TRANS: this is title of dialog window in which user has
 	   to enter path to valid dir */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Output directory"),
-					     /* 2TRANS: this is label in dialog
-						window below it is input field */
-					     _("Enter path to existing output directory"),
-					     &(global_config.audiodir),
-					     CDW_FS_DIR, R_OK|W_OK|X_OK, CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Output directory"),
+				       /* 2TRANS: this is label in dialog
+					  window below it is input field */
+				       _("Enter path to existing output directory"),
+				       &(global_config.audiodir),
+				       CDW_FS_DIR, R_OK|W_OK|X_OK, CDW_FS_EXISTING);
 	cdw_main_window_wrefresh();
 	if (crv != CDW_OK) {
 		/* user failed to provide correct path when
@@ -257,13 +257,13 @@ cdw_rv_t cdw_read_disc_read_data_disc_prepare(cdw_disc_t *disc)
 
 	/* 2TRANS: this is title of dialog window in which user has
 	   to enter path to valid dir */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Output file"),
-					     /* 2TRANS: this is label in dialog
-						window below it is input field */
-					     _("Select new or existing image file"),
-					     &(global_config.general.image_fullpath),
-					     CDW_FS_FILE, R_OK|W_OK,
-					     CDW_FS_NEW | CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Output file"),
+				       /* 2TRANS: this is label in dialog
+					  window below it is input field */
+				       _("Select new or existing image file"),
+				       &(global_config.general.image_fullpath),
+				       CDW_FS_FILE, R_OK|W_OK,
+				       CDW_FS_NEW | CDW_FS_EXISTING);
 	if (crv != CDW_OK) {
 		cdw_vdm ("INFO: file picker returns !CDW_OK\n")
 		return crv;
