@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -215,17 +215,17 @@ cdw_rv_t cdw_erase_wizard(cdw_task_t *task, cdw_disc_t *disc)
 
 cdw_rv_t cdw_erase_wizard_driver(void)
 {
-	int fi = f_cancel_button_b; /* initial focus on "cancel" button */
 	int key = 'a'; /* safe initial value */
+	int fi = f_cancel_button_b; /* Initial focus on "Cancel" button. */
+	cdw_form_driver_focus_on(wizard.cdw_form, fi);
 
 	while (key != CDW_KEY_ESCAPE && key != 'q' && key != 'Q') {
-		key = cdw_form_driver(wizard.cdw_form, fi);
+		key = cdw_form_driver_new(wizard.cdw_form, &fi);
 
 		/* in this wizard driver we are interested only in
 		   ENTER being pressed on either "erase" or "cancel"
 		   buttons */
 		if (key == CDW_KEY_ENTER) {
-			fi = field_index(current_field(wizard.cdw_form->form));
 			if (fi == f_erase_button_b) {
 
 				/* flush */
@@ -418,6 +418,9 @@ cdw_rv_t cdw_erase_wizard_build(cdw_task_t *task, cdw_disc_t *disc)
 		field_opts_off(wizard.cdw_form->fields[f_format_mode_l], O_VISIBLE);
 	}
 
+	/* In newly created wizard the focus should be on Cancel button. */
+	cdw_form_driver_go_to_field(wizard.cdw_form, f_cancel_button_b);
+
 	cdw_form_redraw_widgets(wizard.cdw_form);
 
 	cdw_form_add_return_chars(wizard.cdw_form, CDW_KEY_ENTER, CDW_KEY_ESCAPE, 'q', 'Q', 0);
