@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,19 +27,27 @@
 #include "cdw_form.h"
 #include "cdw_widgets.h"
 #include "cdw_string.h"
+#include "gettext.h"
+
+
+
+
 
 static void cdw_form_driver_focus_off(cdw_form_t *cdw_form, int fi);
 static void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off);
+static void cdw_form_add_return_key_to_widgets(cdw_form_t *form, int key);
 static bool cdw_form_is_return_key(cdw_form_t *form, int key);
+static bool cdw_form_is_movement_key(cdw_form_t *cdw_form, int key);
 
+static bool cdw_form_is_static_label_field_index(cdw_form_t *cdw_form, int fi);
 static bool cdw_form_is_checkbox_field_index(cdw_form_t *cdw_form, int fi);
 static bool cdw_form_is_dropdown_field_index(cdw_form_t *cdw_form, int fi);
 static bool cdw_form_is_button_field_index(cdw_form_t *cdw_form, int fi);
-static bool cdw_form_is_input_field_index(cdw_form_t *cdw_form, int fi);
-static bool cdw_form_is_safe_input_line_index(cdw_form_t *cdw_form, int fi);
-/* To be removed.
-static bool cdw_form_is_checkbox_index(cdw_form_t *cdw_form, int fi);
-*/
+static bool cdw_form_is_text_field_index(cdw_form_t *cdw_form, int fi);
+static bool cdw_form_is_safe_input_line_field_index(cdw_form_t *cdw_form, int fi);
+static bool cdw_form_is_dynamic_label_field_index(cdw_form_t *cdw_form, int fi);
+static bool cdw_form_is_file_selector_field_index(cdw_form_t *cdw_form, int fi);
+
 
 
 
@@ -50,6 +58,8 @@ static cdw_rv_t cdw_form_description_to_dropdown(cdw_form_t *cdw_form, cdw_form_
 static cdw_rv_t cdw_form_description_to_button(cdw_form_t *cdw_form, cdw_form_descr_t *descr, int fi);
 static cdw_rv_t cdw_form_description_to_text(cdw_form_t *cdw_form, cdw_form_descr_t *descr, int fi);
 static cdw_rv_t cdw_form_description_to_dynamic_label(cdw_form_t *cdw_form, cdw_form_descr_t *descr, int fi);
+static cdw_rv_t cdw_form_description_to_file_selector(cdw_form_t *cdw_form, cdw_form_descr_t *descr, int fi);
+
 
 
 
@@ -170,6 +180,10 @@ void cdw_form_delete_form_objects(cdw_form_t *cdw_form)
 				CDW_SAFE_INPUT_LINE *input_line = (CDW_SAFE_INPUT_LINE *) field_userptr(cdw_form->fields[i]);
 				cdw_assert (input_line, "ERROR: field #%d is input_line, but the pointer is null\n", i);
 				cdw_safe_input_line_delete(&input_line);
+			} else if (cdw_form->field_widget_types[i] == CDW_WIDGET_ID_FILE_SELECTOR) {
+				cdw_file_selector_t *selector = (cdw_file_selector_t *) field_userptr(cdw_form->fields[i]);
+				cdw_assert (selector, "ERROR: field #%d is file_selector, but the pointer is null\n", i);
+				cdw_file_selector_delete(&selector);
 			} else {
 				;
 			}
@@ -186,127 +200,133 @@ void cdw_form_delete_form_objects(cdw_form_t *cdw_form)
 
 
 /**
-   \brief New style driver, for widgets that support ->driver() function
+   \brief New style driver (improved), for widgets that support ->driver() function
 
-   This driver is simpler than cdw_form_driver_old(), mainly because it
-   does not handle that many keys. It is only concerned with
-   inter-field movement keys and most important keys: Enter, Escape
-   and Exit.
+   \date Function's top-level comment reviewed on 2016-03-03
+   \date Function's body reviewed on 2016-03-03
 
-   Ideally all forms used by cdw should migrate to this driver.
+   This driver returns (through \p last_fi) index of field/widget that
+   was being operated on as a last one before this function returned.
+   This allows client code to answer a question: "cdw_form_driver()
+   returned a key, which specific widget instance produced this key?".
+
+   So if this form driver returns a key, you know that this key comes
+   from widget that can be accessed through
+   e.g. field_userptr(cdw_form->fields[last_fi]).
+
+   If you want to tell cdw_form from which field to start to operate,
+   use cdw_form_driver_focus_on(cdw_form_t *cdw_form, int fi) before
+   calling this driver function.
 
    In order for this function to work properly, all widgets in given
    \p cdw_form must support ->driver().
 
    \param cdw_form - cdw form to control
-   \param initial_fi - index of widget that will initially have keyboard focus
+   \param last_fi - index of last widget
 
-   \return CDW_KEY_ENTER if one of widgets' driver has returned CDW_KEY_ENTER
-   \return CDW_KEY_ESCAPE if one of widgets' driver has returned CDW_KEY_ESCAPE
-   \return CDW_KEY_EXIT if one of widgets' driver has returned CDW_KEY_EXIT
+   \return key returned by a driver of last visited widget
 */
-int cdw_form_driver(cdw_form_t *cdw_form, int initial_fi)
+int cdw_form_driver_new(cdw_form_t *cdw_form, int *last_fi)
 {
-	cdw_form_driver_go_to_field(cdw_form, initial_fi);
-	/* TODO: why this assignment? */
-	initial_fi = field_index(current_field(cdw_form->form));
-	cdw_form_driver_focus_on(cdw_form, initial_fi);
-
+	int fi = field_index(current_field(cdw_form->form));
+	cdw_form_driver_focus_on(cdw_form, fi);
 
 	int key = 'a'; /* Safe initial value. */
-	int fi = initial_fi;
 
-	while (key != CDW_KEY_ESCAPE && key != CDW_KEY_ENTER && key != KEY_EXIT) {
+	/* Call a widget-specific driver, and don't care about
+	   widget-specific behaviour. The widget-specific behaviour is
+	   defined in widget module.
 
-		/* Call a widget-specific driver, and don't care about
-		   widget-specific behaviour. The widget-specific
-		   behaviour is defined in widget module.
+	   TODO: two widget drivers below accept "cdw_form_t *"
+	   variable as second arg. Consider converting the type of
+	   second arg to all widget's drivers from "void *" to
+	   "cdw_form_t *".
+	*/
+	void *widget = field_userptr(cdw_form->fields[fi]);
+	if (cdw_form_is_button_field_index(cdw_form, fi)) {
+		CDW_BUTTON *b = (CDW_BUTTON *) widget;
+		key = cdw_button_driver(b, (void *) cdw_form);
 
-		   TODO: two widget drivers below accept "cdw_form_t
-		   *" variable as second arg. Consider converting the
-		   type of second arg to all widget's drivers from
-		   "void *" to "cdw_form_t *".
-		*/
-		void *widget = field_userptr(cdw_form->fields[fi]);
-		if (cdw_form_is_button_field_index(cdw_form, fi)) {
-			CDW_BUTTON *b = (CDW_BUTTON *) widget;
-			key = cdw_button_driver(b, (void *) cdw_form);
+	} else if (cdw_form_is_dropdown_field_index(cdw_form, fi)) {
+		CDW_DROPDOWN *dd = (CDW_DROPDOWN *) widget;
+		key = cdw_dropdown_driver(dd, NULL);
 
-		} else if (cdw_form_is_dropdown_field_index(cdw_form, fi)) {
-			CDW_DROPDOWN *dd = (CDW_DROPDOWN *) widget;
-			key = cdw_dropdown_driver(dd, NULL);
+	} else if (cdw_form_is_safe_input_line_field_index(cdw_form, fi)) {
+		CDW_SAFE_INPUT_LINE *line = (CDW_SAFE_INPUT_LINE *) widget;
+		key = cdw_safe_input_line_driver(line, NULL);
 
-		} else if (cdw_form_is_safe_input_line_index(cdw_form, fi)) {
-			CDW_SAFE_INPUT_LINE *line = (CDW_SAFE_INPUT_LINE *) widget;
-			key = cdw_safe_input_line_driver(line, NULL);
+	} else if (cdw_form_is_checkbox_field_index(cdw_form, fi)) {
+		CDW_CHECKBOX *cb = (CDW_CHECKBOX *) widget;
+		key = cdw_checkbox_driver(cb, (void *) cdw_form);
 
-		} else if (cdw_form_is_checkbox_field_index(cdw_form, fi)) {
-			CDW_CHECKBOX *cb = (CDW_CHECKBOX *) widget;
-			key = cdw_checkbox_driver(cb, (void *) cdw_form);
+	} else if (cdw_form_is_file_selector_field_index(cdw_form, fi)) {
+		cdw_file_selector_t *selector = (cdw_file_selector_t *) widget;
+		key = cdw_file_selector_driver(selector);
 
-		} else {
-			/* May be that the only widget in the form is
-			   a widget without a driver (e.g. TEXT
-			   widget). What then? Here is a solution: */
-			key = wgetch(cdw_form->subwindow);
-		}
-
-
-
-		/* Handle movement between widgets. */
-
-
-		if (cdw_form_is_movement_key(key)) {
-			/* We are moving away from a
-			   widget. Un-highlight it to show that
-			   keyboard focus is no longer on the
-			   widget. */
-			int old_fi = field_index(current_field(cdw_form->form));
-			cdw_form_driver_focus_off(cdw_form, old_fi);
-
-
-
-			/* Move to next or previous widget in the form. */
-			if (key == KEY_DOWN || key == CDW_KEY_TAB) {
-				form_driver(cdw_form->form, REQ_NEXT_FIELD);
-				form_driver(cdw_form->form, REQ_END_LINE);
-
-			} else if (key == KEY_UP || key == KEY_BTAB
-				   /* || key == KEY_BACKSPACE */
-				   ) {
-				form_driver(cdw_form->form, REQ_PREV_FIELD);
-				form_driver(cdw_form->form, REQ_END_LINE);
-
-			} else {
-				cdw_vdm ("ERROR: unhandled movement key %d/%s\n", key, cdw_ncurses_key_label(key));
-			}
-
-
-
-			/* We are moving into a next widget. Highlight it to
-			   show that keyboard focus has moved to the
-			   widget. */
-			fi = field_index(current_field(cdw_form->form));
-			cdw_form_driver_focus_on(cdw_form, fi);
-
-
-		} else if (cdw_form_is_return_key(cdw_form, key)) {
-			int ret_fi = field_index(current_field(cdw_form->form));
-			if (isprint(key)
-			    && cdw_form_is_input_field_index(cdw_form, ret_fi)) {
-				/* Keys like 'q'/'Q' may be configured
-				   as return keys, but they shouldn't
-				   be considered as such in regular
-				   text input fields. */
-				;
-			} else {
-				break;
-			}
-		} else {
-			;
-		}
+	} else {
+		/* May be that the only widget in the form is a widget
+		   without a driver (e.g. TEXT widget). What then?
+		   Here is a solution: */
+		key = wgetch(cdw_form->subwindow);
 	}
 
+	/* Record index of field that had focus and now causes this
+	   driver to return a key. This key will come from a
+	   widget. Caller of this function may want to know exactly
+	   which widget returned given key. */
+	*last_fi = fi;
+
+
+	/* Handle movement between widgets. Remember that a movement
+	   key may also be return key (e.g. TAB key).  So do two
+	   separate tests on key: first is_movement_key() and then
+	   is_return_key(). */
+
+	if (cdw_form_is_movement_key(cdw_form, key)) {
+		/* We are moving away from a widget. Un-highlight it
+		   to show that keyboard focus is no longer on the
+		   widget. */
+		cdw_form_driver_focus_off(cdw_form, *last_fi);
+
+
+
+		/* Move to next or previous widget in the form. */
+		if (key == KEY_DOWN || key == CDW_KEY_TAB) {
+			form_driver(cdw_form->form, REQ_NEXT_FIELD);
+			form_driver(cdw_form->form, REQ_END_LINE);
+
+		} else if (key == KEY_UP || key == KEY_BTAB
+			   /* || key == KEY_BACKSPACE */
+			   ) {
+			form_driver(cdw_form->form, REQ_PREV_FIELD);
+			form_driver(cdw_form->form, REQ_END_LINE);
+
+		} else {
+			cdw_vdm ("ERROR: unhandled movement key %d/%s\n", key, cdw_ncurses_key_label(key));
+		}
+
+
+
+		/* We are moving into a next widget. Highlight it to
+		   show that keyboard focus has moved to the
+		   widget. */
+		fi = field_index(current_field(cdw_form->form));
+		cdw_form_driver_focus_on(cdw_form, fi);
+	}
+
+
+	/* Here was the second test: is_return_key(). But read the
+	   following comment: */
+	/* If a widget's driver function decided to return, it is
+	   because parent code (the one that configured form and
+	   widgets) needs to get back control (because of return key
+	   configured for that widget). cdw form shouldn't interfere
+	   with this (shouldn't add another layer of return keys). */
+
+	cdw_vdm ("INFO: returning key %s for field #%d / %s\n",
+		 cdw_ncurses_key_label(key),
+		 *last_fi,
+		 cdw_widgets_widget_type_label(cdw_form->field_widget_types[*last_fi]));
 	return key;
 }
 
@@ -318,6 +338,8 @@ bool cdw_form_is_return_key(cdw_form_t *form, int key)
 {
 	for (int i = 0; i < N_RETURN_KEYS_MAX; i++) {
 		if (form->return_keys[i] == 0) {
+			/* Since the table is non-sparse, this means
+			   that we are after last key in table. */
 			break;
 		} else if (form->return_keys[i] == key) {
 			return true;
@@ -346,6 +368,7 @@ void cdw_form_add_return_char(cdw_form_t *form, int key)
 	} else {
 		form->return_keys[form->n_return_keys++] = key;
 	}
+	cdw_form_add_return_key_to_widgets(form, key);
 
 	return;
 }
@@ -375,6 +398,54 @@ void cdw_form_add_return_chars(cdw_form_t *form, ...)
 
 
 
+
+void cdw_form_add_return_key_to_widgets(cdw_form_t *form, int key)
+{
+	cdw_assert (form, "ERROR: \"form\" argument is NULL\n");
+	cdw_assert (key != 0, "ERROR: \"key\" is zero\n");
+
+	for (int fi = 0; fi < form->n_fields; fi++) {
+
+		cdw_widget_t *widget = NULL;
+
+		if (cdw_form_is_checkbox_field_index(form, fi)) {
+			void *data = field_userptr(form->fields[fi]);
+			CDW_CHECKBOX *cb = (CDW_CHECKBOX *) data;
+			widget = &cb->widget;
+		} else if (cdw_form_is_dropdown_field_index(form, fi)) {
+			void *data = field_userptr(form->fields[fi]);
+			CDW_DROPDOWN *dd = (CDW_DROPDOWN *) data;
+			widget = &dd->widget;
+		} else if (cdw_form_is_button_field_index(form, fi)) {
+			void *data = field_userptr(form->fields[fi]);
+			CDW_BUTTON *b = (CDW_BUTTON *) data;
+			widget = &b->widget;
+		} else if (cdw_form_is_safe_input_line_field_index(form, fi)) {
+			void *data = field_userptr(form->fields[fi]);
+			CDW_SAFE_INPUT_LINE *line = (CDW_SAFE_INPUT_LINE *) data;
+			widget = &line->widget;
+		} else if (cdw_form_is_file_selector_field_index(form, fi)) {
+			void *data = field_userptr(form->fields[fi]);
+			cdw_file_selector_t *fs = (cdw_file_selector_t *) data;
+			widget = &fs->widget;
+		} else {
+			continue;
+		}
+
+		cdw_assert (widget, "ERROR: widget not found\n");
+		cdw_assert (widget->add_return_key, "ERROR: add_return_key function not found\n");
+
+		if (widget && widget->add_return_key) {
+			widget->add_return_key(widget, key);
+		}
+	}
+	return;
+}
+
+
+
+
+
 typedef cdw_rv_t (* cdw_form_converter_t)(cdw_form_t *cdw_form, cdw_form_descr_t *descr, int fi);
 
 
@@ -388,7 +459,8 @@ cdw_form_converter_t cdw_form_converters[] = {
 	cdw_form_description_to_button,
 	cdw_form_description_to_text,
 	cdw_form_description_to_safe_input_line,
-	cdw_form_description_to_dynamic_label
+	cdw_form_description_to_dynamic_label,
+	cdw_form_description_to_file_selector
 };
 
 
@@ -673,18 +745,83 @@ cdw_rv_t cdw_form_description_to_dynamic_label(cdw_form_t *cdw_form, cdw_form_de
 
 
 
-bool cdw_form_is_movement_key(int key)
+cdw_rv_t cdw_form_description_to_file_selector(cdw_form_t *cdw_form, cdw_form_descr_t *descr, int fi)
 {
-	if (key == KEY_DOWN
-	    || key == CDW_KEY_TAB
-	    || key == KEY_UP
-	    || key == KEY_BTAB
+	cdw_vdm ("INFO: initial string = \"%s\"\n", (char *) descr->data1);
+	cdw_form->fields[fi] = cdw_ncurses_new_input_field(descr->n_lines,
+							   descr->n_cols,
+							   descr->begin_y,
+							   descr->begin_x,
+							   (char *) NULL,
+							   0,
+							   CDW_NCURSES_INPUT_NONE,
+							   CDW_COLORS_INPUT);
 
-	    /* in some configurations backspace may be a movement key,
-	       i.e. when cursor is at the beginning of input field */
-	    /* || key == KEY_BACKSPACE */
-	    ) {
+	if (!cdw_form->fields[fi]) {
+		cdw_vdm ("ERROR: failed to initialize field #%d\n", fi);
+		return CDW_ERROR;
+	}
 
+	cdw_file_selector_t *selector = cdw_file_selector_new();
+	if (!selector) {
+		cdw_vdm ("ERROR: failed to create file selector widget for field #%d\n", fi);
+		return CDW_ERROR;
+	}
+
+	cdw_rv_t crv = cdw_file_selector_show(selector,
+					      cdw_form->subwindow,
+					      descr->n_lines, descr->n_cols, descr->begin_y, descr->begin_x,
+					      CDW_COLORS_INPUT,
+					      /* 2TRANS: this is a title of file selector window, a noun. */
+					      _("File selector"),      /* Title. */
+					      (char *) descr->data1);  /* Fullpath. */
+
+	if (crv != CDW_OK) {
+		cdw_vdm ("ERROR: failed to show file selector widget for field #%d\n", fi);
+		return CDW_ERROR;
+	}
+
+	int rv = set_field_userptr(cdw_form->fields[fi], (void *) selector);
+	if (rv != E_OK) {
+		cdw_vdm ("ERROR: can't set userptr for field #%d: %d (%s) \n",
+			 fi, rv, cdw_ncurses_error_string(rv));
+		return CDW_ERROR;
+	}
+
+	cdw_form->field_widget_types[fi] = CDW_WIDGET_ID_FILE_SELECTOR;
+	return CDW_OK;
+}
+
+
+
+
+
+bool cdw_form_is_movement_key(cdw_form_t *cdw_form, int key)
+{
+	if (key == KEY_DOWN || key == KEY_UP) {
+
+		int fi = field_index(current_field(cdw_form->form));
+		if (cdw_form_is_file_selector_field_index(cdw_form, fi)) {
+			/* These keys in file selector aren't supposed
+			   to cause movement between widgets.
+
+			   They may be added to file selector as
+			   return keys because selector's parent code
+			   may want to read current file. Ignore the
+			   keys in context of movement between
+			   widgets.
+
+			   TODO: this should be true for all widgets
+			   based on list display widget, not only for
+			   file selector. But for now only a file
+			   selector may be embedded in cdw form. */
+
+			cdw_vdm ("INFO: %s ignored as form movement key in list display\n", cdw_ncurses_key_label(key));
+			return false;
+		} else {
+			return true;
+		}
+	} else if (key == CDW_KEY_TAB || key == KEY_BTAB) {
 		return true;
 	} else {
 		return false;
@@ -764,14 +901,24 @@ void cdw_form_driver_toggle_focus(cdw_form_t *cdw_form, int fi, bool off)
 			function(dropdown);
 		}
 	} else if (cdw_form_is_checkbox_field_index(cdw_form, fi)
-		   || cdw_form_is_input_field_index(cdw_form, fi)) {
-		if (off) {
-			/* moving away from checkbox/input, don't display blinking cursor */
-			curs_set(0);
-		} else {
-			/* moving away from checkbox/input, display blinking cursor */
-			curs_set(1);
-		}
+		   || cdw_form_is_safe_input_line_field_index(cdw_form, fi)) {
+
+		curs_set(off ? 0 : 1);
+
+	} else if (cdw_form_is_static_label_field_index(cdw_form, fi)
+		   || cdw_form_is_text_field_index(cdw_form, fi)
+		   || cdw_form_is_dynamic_label_field_index(cdw_form, fi)
+		   || cdw_form_is_file_selector_field_index(cdw_form, fi)) {
+
+		/* Most of them are text areas that don't accept/allow
+		   text input from keyboard when focus is on them. Similarly for file
+		   selector widget.
+
+		   Of course normally cdw form driver won't go into
+		   text fields, but if such a field is the only field
+		   in a form, we don't want to display cursor when
+		   focus is on such a field. */
+		curs_set(off ? 1 : 0);
 	} else {
 		;
 	}
@@ -801,9 +948,9 @@ CDW_DROPDOWN *cdw_form_get_dropdown(cdw_form_t *cdw_form, int fi)
 
 	CDW_DROPDOWN *dd = (CDW_DROPDOWN *) field_userptr(cdw_form->fields[fi]);
 	cdw_assert (dd, "ERROR: field index %d: field user pointer is NULL\n", fi);
-	cdw_assert (dd->widget_id == CDW_WIDGET_ID_DROPDOWN,
+	cdw_assert (dd->widget.type_id == CDW_WIDGET_ID_DROPDOWN,
 		    "ERROR: field index %d: internal widget id is not DROPDOWN, is %s\n",
-		    fi, cdw_widgets_widget_type_label(dd->widget_id));
+		    fi, cdw_widgets_widget_type_label(dd->widget.type_id));
 
 	return dd;
 }
@@ -827,9 +974,9 @@ CDW_DYNAMIC_LABEL *cdw_form_get_dynamic_label(cdw_form_t *cdw_form, int fi)
 
 	CDW_DYNAMIC_LABEL *label = (CDW_DYNAMIC_LABEL *) field_userptr(cdw_form->fields[fi]);
 	cdw_assert (label, "ERROR: field index %d: field user pointer is NULL\n", fi);
-	cdw_assert (label->widget_id == CDW_WIDGET_ID_DYNAMIC_LABEL,
+	cdw_assert (label->widget.type_id == CDW_WIDGET_ID_DYNAMIC_LABEL,
 		    "ERROR: field index %d: internal widget id is not DYNAMIC_LABEL, is %s\n",
-		    fi, cdw_widgets_widget_type_label(label->widget_id));
+		    fi, cdw_widgets_widget_type_label(label->widget.type_id));
 
 	return label;
 }
@@ -853,9 +1000,9 @@ CDW_SAFE_INPUT_LINE *cdw_form_get_safe_input_line(cdw_form_t *cdw_form, int fi)
 
 	CDW_SAFE_INPUT_LINE *line = (CDW_SAFE_INPUT_LINE *) field_userptr(cdw_form->fields[fi]);
 	cdw_assert (line, "ERROR: field index %d: field user pointer is NULL\n", fi);
-	cdw_assert (line->widget_id == CDW_WIDGET_ID_SAFE_INPUT_LINE,
+	cdw_assert (line->widget.type_id == CDW_WIDGET_ID_SAFE_INPUT_LINE,
 		    "ERROR: field index %d: internal widget id is not SAFE_INPUT_LINE, is %s\n",
-		    fi, cdw_widgets_widget_type_label(line->widget_id));
+		    fi, cdw_widgets_widget_type_label(line->widget.type_id));
 
 	return line;
 }
@@ -918,9 +1065,9 @@ CDW_BUTTON *cdw_form_get_button(cdw_form_t *cdw_form, int fi)
 
 	CDW_BUTTON *button = (CDW_BUTTON *) field_userptr(cdw_form->fields[fi]);
 	cdw_assert (button, "ERROR: field index %d: field user pointer is NULL\n", fi);
-	cdw_assert (button->widget_id == CDW_WIDGET_ID_BUTTON,
+	cdw_assert (button->widget.type_id == CDW_WIDGET_ID_BUTTON,
 		    "ERROR: field index %d: internal widget id is not BUTTON, is %s\n",
-		    fi, cdw_widgets_widget_type_label(button->widget_id));
+		    fi, cdw_widgets_widget_type_label(button->widget.type_id));
 
 	return button;
 }
@@ -944,9 +1091,9 @@ CDW_CHECKBOX *cdw_form_get_checkbox(cdw_form_t *cdw_form, int fi)
 
 	CDW_CHECKBOX *cb = (CDW_CHECKBOX *) field_userptr(cdw_form->fields[fi]);
 	cdw_assert (cb, "ERROR: field index %d: field user pointer is NULL\n", fi);
-	cdw_assert (cb->widget_id == CDW_WIDGET_ID_CHECKBOX,
+	cdw_assert (cb->widget.type_id == CDW_WIDGET_ID_CHECKBOX,
 		    "ERROR: field index %d: internal widget id is not CHECKBOX, is %s\n",
-		    fi, cdw_widgets_widget_type_label(cb->widget_id));
+		    fi, cdw_widgets_widget_type_label(cb->widget.type_id));
 
 	return cb;
 }
@@ -955,6 +1102,45 @@ CDW_CHECKBOX *cdw_form_get_checkbox(cdw_form_t *cdw_form, int fi)
 
 
 
+cdw_file_selector_t *cdw_form_get_file_selector(cdw_form_t *cdw_form, int fi)
+{
+	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
+		    "ERROR: field index %d is out of range (n_fields = %d)\n",
+		    fi, cdw_form->n_fields);
+
+	cdw_assert (cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_FILE_SELECTOR,
+		    "ERROR: field index %d: widget type is not FILE_SELECTOR, is %s\n",
+		    fi, cdw_widgets_widget_type_label(cdw_form->field_widget_types[fi]));
+
+	cdw_assert (cdw_form->fields[fi], "ERROR: field %d is NULL\n", fi);
+
+
+	cdw_file_selector_t *selector = (cdw_file_selector_t *) field_userptr(cdw_form->fields[fi]);
+	cdw_assert (selector, "ERROR: field index %d: field user pointer is NULL\n", fi);
+	cdw_assert (selector->widget.type_id == CDW_WIDGET_ID_FILE_SELECTOR,
+		    "ERROR: field index %d: internal widget id is not FILE_SELECTOR, is %s\n",
+		    fi, cdw_widgets_widget_type_label(selector->widget.type_id));
+
+	return selector;
+}
+
+
+
+
+
+bool cdw_form_is_static_label_field_index(cdw_form_t *cdw_form, int fi)
+{
+	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
+		    "ERROR: field index %d is out of range (n_fields = %d)\n",
+		    fi, cdw_form->n_fields);
+
+	return cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_STATIC_LABEL;
+}
+
+
+
+
+
 bool cdw_form_is_checkbox_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
@@ -981,7 +1167,7 @@ bool cdw_form_is_dropdown_field_index(cdw_form_t *cdw_form, int fi)
 
 
 
-bool cdw_form_is_safe_input_line_index(cdw_form_t *cdw_form, int fi)
+bool cdw_form_is_safe_input_line_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
 		    "ERROR: field index %d is out of range (n_fields = %d)\n",
@@ -993,16 +1179,29 @@ bool cdw_form_is_safe_input_line_index(cdw_form_t *cdw_form, int fi)
 
 
 
-#if 0 /* To be removed. */
-bool cdw_form_is_checkbox_index(cdw_form_t *cdw_form, int fi)
+
+bool cdw_form_is_dynamic_label_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
 		    "ERROR: field index %d is out of range (n_fields = %d)\n",
 		    fi, cdw_form->n_fields);
 
-	return cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_CHECKBOX;
+	return cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_DYNAMIC_LABEL;
 }
-#endif
+
+
+
+
+
+bool cdw_form_is_file_selector_field_index(cdw_form_t *cdw_form, int fi)
+{
+	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
+		    "ERROR: field index %d is out of range (n_fields = %d)\n",
+		    fi, cdw_form->n_fields);
+
+	return cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_FILE_SELECTOR;
+}
+
 
 
 
@@ -1020,13 +1219,13 @@ bool cdw_form_is_button_field_index(cdw_form_t *cdw_form, int fi)
 
 
 
-bool cdw_form_is_input_field_index(cdw_form_t *cdw_form, int fi)
+bool cdw_form_is_text_field_index(cdw_form_t *cdw_form, int fi)
 {
 	cdw_assert (fi >= 0 && fi < cdw_form->n_fields,
 		    "ERROR: field index %d is out of range (n_fields = %d)\n",
 		    fi, cdw_form->n_fields);
 
-	return cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_SAFE_INPUT_LINE;
+	return cdw_form->field_widget_types[fi] == CDW_WIDGET_ID_TEXT;
 }
 
 
@@ -1037,6 +1236,7 @@ void cdw_form_driver_go_to_field(cdw_form_t *cdw_form, int fi)
 {
 	FORM *form = cdw_form->form;
 	if (fi < 0 || fi >= cdw_form->n_fields) {
+		/* TODO: here should be an assert. */
 		cdw_vdm ("WARNING: fi = %d out of bounds (n_fields = %d)\n", fi, cdw_form->n_fields);
 		fi = 0;
 	}
@@ -1054,6 +1254,9 @@ void cdw_form_driver_go_to_field(cdw_form_t *cdw_form, int fi)
 	if (i == fi) {
 		;
 	} else {
+		cdw_form_driver_focus_off(cdw_form,
+					  field_index(current_field(cdw_form->form)));
+
 		while (i < fi && i < cdw_form->n_fields - 1) {
 			rv = form_driver(form, REQ_NEXT_FIELD);
 			if (rv != E_OK) {
@@ -1090,6 +1293,9 @@ void cdw_form_redraw_widgets(cdw_form_t *cdw_form)
 		} else if (id == CDW_WIDGET_ID_DROPDOWN) {
 			CDW_DROPDOWN *dd = cdw_form_get_dropdown(cdw_form, i);
 			cdw_dropdown_display_current_item(dd);
+		} else if (id == CDW_WIDGET_ID_FILE_SELECTOR) {
+			cdw_file_selector_t *selector = cdw_form_get_file_selector(cdw_form, i);
+			cdw_file_selector_refresh(selector);
 		} else {
 			;
 		}
@@ -1252,11 +1458,11 @@ cdw_rv_t cdw_form_replace_field(cdw_form_t *cdw_form, cdw_form_descr_t *descr, i
 
 	   So first unpost form, then free form, then free field. */
 	int rv = unpost_form(cdw_form->form);
-	cdw_vdm_n ("unpost_form", rv, "");
+	cdw_vdm_n2 ("unpost_form", rv, "");
 	rv = free_form(cdw_form->form);
-	cdw_vdm_n ("free_form", rv, "");
+	cdw_vdm_n2 ("free_form", rv, "");
 	rv = free_field(cdw_form->fields[fi]);
-	cdw_vdm_n ("free_field", rv, "");
+	cdw_vdm_n2 ("free_field", rv, "");
 
 
 	/* Create and insert new widget. */
