@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -177,7 +177,7 @@ void display_tooltip(int item)
 			break;
 		case CDW_MENU_RIP_DISC:
 			/* 2TRANS: this is tooltip displayed at the bottom of main cdw window */
-			cdw_window_print_message(subwindow, _("Copy content of optical disc to to your hard drive."), CDW_ALIGN_LEFT);
+			cdw_window_print_message(subwindow, _("Copy content of optical disc to your hard drive."), CDW_ALIGN_LEFT);
 			break;
 		case CDW_MENU_BURN_FILES:
 			/* 2TRANS: this is tooltip displayed at the bottom of main cdw window */
@@ -212,7 +212,7 @@ void display_tooltip(int item)
 			break;
 	}
 	int rv = wrefresh(subwindow);
-	cdw_vdm_n ("wrefresh", rv, "");
+	cdw_vdm_n2 ("wrefresh", rv, "");
 
 	return;
 }
