@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -270,11 +270,12 @@ cdw_rv_t cdw_image_wizard_init(cdw_task_t *task)
 
 int cdw_image_wizard_driver(void)
 {
-	int fi = f_cancel_b; /* Initial focus on "Cancel" button. */
 	int key = 'a';
+	int fi = f_cancel_b; /* Initial focus on "Cancel" button. */
+	cdw_form_driver_focus_on(wizard.cdw_form, fi);
 
 	while (key != CDW_KEY_ESCAPE && key != 'q' && key != 'Q') {
-		key = cdw_form_driver(wizard.cdw_form, fi);
+		key = cdw_form_driver_new(wizard.cdw_form, &fi);
 
 		/* In top-level driver of wizard we are mainly
 		   interested in ENTER being pressed on either
@@ -282,7 +283,6 @@ int cdw_image_wizard_driver(void)
 		   on "Cancel" button, form driver will return ESCAPE
 		   key. */
 		if (key == CDW_KEY_ENTER) {
-			fi = field_index(current_field(wizard.cdw_form->form));
 			if (fi != f_create_b) {
 				/* We are not on "Create" button, so do nothing, wait for next key. */
 				continue;
@@ -438,6 +438,11 @@ cdw_rv_t cdw_image_wizard_build(cdw_task_t *task)
 	   input field of appropriate width. */
 	cdw_image_wizard_recreate_volume_id_input(wizard.cdw_form);
 
+	/* Call to cdw_image_wizard_recreate_volume_id_input may have
+	   moved focus to first dropdown in wizard. We don't want that
+	   in freshly built wizard. */
+	cdw_form_driver_go_to_field(wizard.cdw_form, f_cancel_b);
+
 	/* Make sure that cdw widgets are presented properly. */
 	cdw_form_redraw_widgets(wizard.cdw_form);
 
@@ -571,13 +576,13 @@ int cdw_image_wizard_options_window_show(cdw_id_t *pid, int *fi)
 int cdw_image_wizard_image_file_selector(__attribute__((unused)) void *dummy1, __attribute__((unused)) void *dummy2)
 {
 	/* 2TRANS: this is title of dialog window */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Path to image file"),
-					     /* 2TRANS: this is message in dialog window;
-					     below it there is an input field where user can
-					     enter path to ISO9660 or UDF image file */
-					     _("Please enter FULL path to new image file:"),
-					     &(wizard.task->image_file_fullpath),
-					     CDW_FS_FILE, R_OK | W_OK, CDW_FS_NEW | CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Path to image file"),
+				       /* 2TRANS: this is message in dialog window;
+					  below it there is an input field where user can
+					  enter path to ISO9660 or UDF image file */
+				       _("Please enter FULL path to new image file:"),
+				       &(wizard.task->image_file_fullpath),
+				       CDW_FS_FILE, R_OK | W_OK, CDW_FS_NEW | CDW_FS_EXISTING);
 
 	cdw_main_window_wrefresh();
 
@@ -1267,19 +1272,29 @@ cdw_rv_t cdw_image_wizard_validate_and_save(cdw_task_t *task, int *fi)
 	} else {
 		cdw_vdm ("INFO: canonicalized fullpath is \"%s\"\n", canonical_fullpath);
 
-		int fs_check = cdw_fs_check_fullpath(canonical_fullpath, CDW_FS_FILE, W_OK | R_OK, CDW_FS_EXISTING);
+		int e = cdw_fs_check_fullpath(canonical_fullpath, CDW_FS_FILE, W_OK | R_OK, CDW_FS_NEW | CDW_FS_EXISTING);
 
 		free(canonical_fullpath);
 		canonical_fullpath = (char *) NULL;
-		if (fs_check & CDW_FS_CHECK_EXISTS) {
-			cdw_rv_t eh = cdw_fs_errno_handler(EEXIST);
-			if (eh == CDW_OK) {
-				; /* Pass to next check. */
-			} else {
-				/* Don't overwrite file, allow user to modify file fullpath. */
-				*fi = f_image_file_i;
-				return CDW_NO;
-			}
+
+		if (e == 0) {
+			/* Path is OK, go to saving path. */
+		} else if (e == EEXIST) {
+			/* This case Has been already handled in
+			   cdw_fs_check_fullpath() using
+			   cdw_fs_errno_handler(). e == EEXIST means
+			   that user doesn't want to overwrite the
+			   file.
+
+			   Don't overwrite file, allow user to modify
+			   file fullpath. */
+			*fi = f_image_file_i;
+			return CDW_NO;
+		} else {
+			/* All other errno values. */
+			cdw_fs_errno_handler(e);
+			*fi = f_image_file_i;
+			return CDW_NO;
 		}
 	}
 
