@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,10 +73,13 @@
 #include "cdw_debug.h"
 #include "cdw_list_display.h"
 #include "cdw_ncurses.h"
+#include "cdw_config.h"
 
 
-static void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offset, bool highlight);
-static int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset);
+static void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, bool highlight);
+static int  cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display);
+static bool cdw_list_display_is_lscroll_key(int key);
+static bool cdw_list_display_is_rscroll_key(int key);
 
 
 /**
@@ -155,6 +158,8 @@ CDW_LIST_DISPLAY *cdw_list_display_new(WINDOW *parent, int n_lines, int n_cols,
 		return (CDW_LIST_DISPLAY *) NULL;
 	}
 
+	display->h_offset = 0;
+
 	display->display_format = CDW_LIST_DISPLAY_FORMAT_SHORT;
 	display->display_item = NULL;
 	display->n_items = 0;
@@ -269,24 +274,22 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 	cdw_assert (display, "ERROR: \"display\" argument in NULL\n");
 	cdw_assert (display->display_item, "ERROR: the display doesn't have ->display_item() set\n");
 
-	size_t h_offset = 0; /* indicates if and how much content of the window was moved to left */
-
 	if (display->n_items != 0) {
 		/* initialization: highlight initial item */
 		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
 		cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
-		display->display_item(display, item->data, subwindow_row, h_offset, true);
+		display->display_item(display, item->data, subwindow_row, display->h_offset, true);
 	}
 	wrefresh(display->subwindow);
 
-	int key = cdw_list_display_driver_loop(display, &h_offset);
+	int key = cdw_list_display_driver_loop(display);
 
 	if (display->n_items != 0) {
 		/* control (and focus) is leaving display area, and to indicate this,
 		   we un-highlight current item */
 		cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
 		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
-		display->display_item(display, item->data, subwindow_row, h_offset, false);
+		display->display_item(display, item->data, subwindow_row, display->h_offset, false);
 	}
 
 	wrefresh(display->subwindow);
@@ -298,7 +301,7 @@ int cdw_list_display_driver(CDW_LIST_DISPLAY *display)
 
 
 
-int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
+int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display)
 {
 	int key = 'a'; /* safe (?) initial value */
 	while ((key = wgetch(display->window)) != ERR) {
@@ -307,14 +310,14 @@ int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
 				/* first Home key should move content of
 				   window to beginning of lines, leaving
 				   vertical position unchanged */
-				if (*h_offset > 0) {
-					*h_offset = 0;
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+				if (display->h_offset > 0) {
+					display->h_offset = 0;
+					cdw_list_display_scroll_to_current(display, true);
 				} else {
 					if (display->current_item_ind > 0) {
 						display->current_item_ind = 0;
-						*h_offset = 0;
-						cdw_list_display_scroll_to_current(display, *h_offset, true);
+						display->h_offset = 0;
+						cdw_list_display_scroll_to_current(display, true);
 					}
 				}
 				break;
@@ -323,40 +326,54 @@ int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
 				/* go to last item */
 				if (display->current_item_ind < display->n_items - 1) {
 					display->current_item_ind = display->n_items - 1;
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+					cdw_list_display_scroll_to_current(display, true);
 				}
 				break;
 
 			case KEY_DOWN:
-				/* go to next item */
+				/* Go down to next item. */
 				if (display->current_item_ind < display->n_items - 1) {
 					display->current_item_ind++;
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+					cdw_list_display_scroll_to_current(display, true);
 				}
 				break;
 
 			case KEY_UP:
-				/* go to previous item */
+				/* Go up to previous item. */
 				if (display->current_item_ind > 0) {
 					display->current_item_ind--;
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+					cdw_list_display_scroll_to_current(display, true);
 				}
 
 				break;
 
 			case KEY_RIGHT:
-				/* some modules using list display may use
-				   "item display function" that does not
-				   support non-zero h_offset, it will be
-				   ignored by such "item display function" */
-				(*h_offset)++;
-				cdw_list_display_scroll_to_current(display, *h_offset, true);
+		        case '>':
+				/* Scroll horizontally to right. */
+				/* Some modules using list display may
+				   suppress KEY_RIGHT/KEY_LEFT and use
+				   them for other purposes. */
+				if (cdw_list_display_is_rscroll_key(key)
+				    && display->h_offset < CDW_LIST_DISPLAY_H_OFFSET_LIMIT) {
+
+					/* Some modules using list display may use
+					   "item display function" that does not
+					   support non-zero h_offset, it will be
+					   ignored by such "item display function". */
+					display->h_offset++;
+					cdw_list_display_scroll_to_current(display, true);
+				}
 				break;
+
 			case KEY_LEFT:
-				/* see comment for KEY_RIGHT */
-				if (*h_offset > 0) {
-					(*h_offset)--;
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+		        case '<':
+				/* Scroll horizontally to left. */
+				/* See comment for KEY_RIGHT. */
+				if (cdw_list_display_is_lscroll_key(key)) {
+					if (display->h_offset > 0) {
+						display->h_offset--;
+						cdw_list_display_scroll_to_current(display, true);
+					}
 				}
 				break;
 
@@ -373,7 +390,7 @@ int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
 						   screen size up */
 						display->current_item_ind -= (size_t) display->n_lines;
 					}
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+					cdw_list_display_scroll_to_current(display, true);
 				}
 				break;
 
@@ -384,7 +401,7 @@ int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
 					if (display->current_item_ind > display->n_items - 1) {
 						display->current_item_ind = display->n_items - 1;
 					}
-					cdw_list_display_scroll_to_current(display, *h_offset, true);
+					cdw_list_display_scroll_to_current(display, true);
 				}
 				break;
 
@@ -408,6 +425,54 @@ int cdw_list_display_driver_loop(CDW_LIST_DISPLAY *display, size_t *h_offset)
 
 
 
+/**
+   \brief Should given key result in scroll to right?
+
+   \param key
+
+   \return true if a list display should be scrolled right when a \p key is pressed
+   \return false otherwise
+*/
+bool cdw_list_display_is_rscroll_key(int key)
+{
+	if (key == '>') {
+		return true;
+	}
+
+	if (key == KEY_RIGHT && !cdw_config_fs_lynx_like_motion()) {
+		return true;
+	}
+
+	return false;
+}
+
+
+
+
+
+/**
+   \brief Should given key result in scroll to left?
+
+   \param key
+
+   \return true if a list display should be scrolled left when a \p key is pressed
+   \return false otherwise
+*/
+bool cdw_list_display_is_lscroll_key(int key)
+{
+	if (key == '<') {
+		return true;
+	}
+
+	if (key == KEY_LEFT && !cdw_config_fs_lynx_like_motion()) {
+		return true;
+	}
+
+	return false;
+}
+
+
+
 
 /**
    \brief Refresh windows associated with given display
@@ -426,6 +491,8 @@ void cdw_list_display_refresh(CDW_LIST_DISPLAY *display)
 	redrawwin(display->window);
 	wrefresh(display->window);
 
+	cdw_list_display_scroll_to_current(display, false);
+
 	return;
 }
 
@@ -441,25 +508,17 @@ void cdw_list_display_refresh(CDW_LIST_DISPLAY *display)
    display's subwindow. If \p highlight is true, then the item is additionally
    highlighted.
 
-   \p h_offset indicates how many columns to left should the lines in the display
-   be moved. This is useful when displayed item is represented by line with more
-   chars than there is columns in display's subwindow - the line should be moved
-   (shifted) to left to display content truncated by display's border. This value
-   affects all lines displayed in the display, however: read next paragraph.
-
    This function doesn't draw representation of item itself. It uses pointer to
    a function: display->display_item ("item display function"), which must be
    set and provided by code using the display (set by display->display_item).
-   "item display function" is allowed not to implement support for h_offset,
+   "item display function" is allowed not to implement support for display->h_offset,
    and to ignore its value.
 
    \param display - display, in which given item exists, and which you want to
                     use to display the item
-   \param h_offset - value of horizontal offset, with which lines should be
-                     displayed in the display
    \param highlight - should an item indicated by \p item_i be highlighted?
 */
-void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offset, bool highlight)
+void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, bool highlight)
 {
 	cdw_assert (display, "ERROR: \"display\" argument is NULL\n");
 	cdw_assert (display->subwindow, "ERROR: \"display->subwindow\" is NULL\n");
@@ -487,7 +546,7 @@ void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offs
 	     subwindow_row < display->n_lines && item != (cdw_dll_item_t *) NULL;
 	     subwindow_row++, item = item->next, item_ind++) {
 
-		display->display_item(display, item->data, (size_t) subwindow_row, h_offset, false);
+		display->display_item(display, item->data, (size_t) subwindow_row, display->h_offset, false);
 
 		/* since we already traverse list of items, we can update
 		   value of "current_item" field: since this function was called,
@@ -502,7 +561,7 @@ void cdw_list_display_scroll_to_current(CDW_LIST_DISPLAY *display, size_t h_offs
 		/* highlight current item */
 		item = cdw_dll_ith_item(display->list, display->current_item_ind);
 		size_t subwindow_row = display->current_item_ind % (size_t) display->n_lines;
-		display->display_item(display, item->data, subwindow_row, h_offset, true);
+		display->display_item(display, item->data, subwindow_row, display->h_offset, true);
 	}
 
 	wrefresh(display->subwindow);
@@ -538,7 +597,7 @@ void cdw_list_display_add_return_key(CDW_LIST_DISPLAY *display, int c)
 	cdw_assert (c != 0, "ERROR: trying to add char == 0, but 0 is an initializer value\n");
 
 	if (cdw_list_display_is_return_key(display, c)) {
-		cdw_vdm ("WARNING: attempting to add char %d / \"%s\", but it is already on the list of return chars\n",
+		cdw_sdm ("WARNING: attempting to add char %d / \"%s\", but it is already on the list of return chars\n",
 			 c, cdw_ncurses_key_label(c));
 	} else {
 		display->return_chars[display->n_return_chars++] = c;
@@ -621,7 +680,7 @@ bool cdw_list_display_is_return_key(CDW_LIST_DISPLAY *display, int c)
 /**
    \brief Remove i-th item from given display
 
-   Function looks up \p item_i-th item in given display, removes it from
+   Function looks up \p i-th item in given display, removes it from
    associated window and associated list. It also updates value of 'number
    of items' variable associated with the display.
 
@@ -632,21 +691,19 @@ bool cdw_list_display_is_return_key(CDW_LIST_DISPLAY *display, int c)
    function.
 
    \param display - display from which to remove an item
-   \param item_i - index of item to be removed
+   \param i - index of item to be removed
 
    \return CDW_OK if removing was successfull
-   \return CDW_GEN_ERROR if it failed
+   \return CDW_ERROR if it failed
 */
-cdw_rv_t cdw_list_display_remove_item(CDW_LIST_DISPLAY *display, size_t item_i)
+cdw_rv_t cdw_list_display_remove_item(CDW_LIST_DISPLAY *display, size_t i)
 {
 	cdw_assert (display, "ERROR: display is null\n");
 	cdw_assert (display->n_items > 0, "ERROR: you called the function for display with empty list\n");
 	cdw_assert (display->list != (cdw_dll_item_t *) NULL, "ERROR: list associated with the display is null\n");
 
-	display->current_item_ind = item_i;
-
 #ifndef NDEBUG
-	cdw_dll_item_t *item = cdw_dll_ith_item(display->list, display->current_item_ind);
+	cdw_dll_item_t *item = cdw_dll_ith_item(display->list, i);
 	cdw_assert (!item->data, "ERROR: item payload is not NULL, trying to leak memory?\n");
 #endif
 
@@ -654,35 +711,35 @@ cdw_rv_t cdw_list_display_remove_item(CDW_LIST_DISPLAY *display, size_t item_i)
 	size_t len1 = cdw_dll_length(display->list);
 #endif
 
-	cdw_rv_t crv = cdw_dll_remove_ith_item(&(display->list), display->current_item_ind);
+	cdw_rv_t crv = cdw_dll_remove_ith_item(&(display->list), i);
 
 #ifndef NDEBUG
 	size_t len2 = cdw_dll_length(display->list);
 #endif
 
 	if (crv == CDW_OK) {
-		display->n_items--; /* to avoid calling cdw_dll_length() */
+		display->n_items--; /* Decrement is faster than call to cdw_dll_length(). */
 
 		cdw_assert (len2 == len1 - 1, "ERROR: lengths of list do not differ by one: len before: %zd, len after: %zd\n", len1, len2);
 		cdw_assert (len2 == display->n_items, "ERROR: length mismatch\n");
 
 		if (display->n_items > 0) {
-			if (display->current_item_ind == 0) {
-				; /* removed item was first, no need to
-				     reposition cursor in new list; new
-				     current item will also be first */
-			} else if (display->current_item_ind == display->n_items) {
-				/* removed item was last one;
-				   here n_files has new, decreased value */
+			if (display->current_item_ind == display->n_items) {
+				/* Removed item was at the bottom of
+				   list, so reposition the index.
+				   Notice that ->n_items in the
+				   condition has new, decremented
+				   value. */
 				display->current_item_ind = display->n_items - 1;
-				/* we could have achieved the same effect
-				   by simple display->current_item_ind--; */
 			} else {
-				; /* removed item was somewhere in between
-				     beginning and end, no need to change item_i */
+				/* display->current_item_ind is still
+				   a valid index of *some* non-deleted
+				   list item, so no need to change
+				   it. */
+				;
 			}
 
-			cdw_list_display_scroll_to_current(display, 0, true);
+			cdw_list_display_scroll_to_current(display, true);
 		} else {
 			werase(display->subwindow);
 			wrefresh(display->subwindow);
@@ -740,7 +797,14 @@ void cdw_list_display_scroll_to(CDW_LIST_DISPLAY *display, size_t current_item_i
 		cdw_assert (current_item_ind < display->n_items, "ERROR: item index %zd is larger than number of items %zd\n",
 			    current_item_ind, display->n_items);
 		display->current_item_ind = current_item_ind;
-		cdw_list_display_scroll_to_current(display, h_offset, highlight);
+
+		if (h_offset > CDW_LIST_DISPLAY_H_OFFSET_LIMIT) {
+			display->h_offset = CDW_LIST_DISPLAY_H_OFFSET_LIMIT;
+		} else {
+			display->h_offset = h_offset;
+		}
+
+		cdw_list_display_scroll_to_current(display, highlight);
 	} else {
 		cdw_vdm ("%s: called the function for empty list, requested_item_ind = %zd\n",
 			 current_item_ind == 0 ? "WARNING" : "ERROR", current_item_ind);
@@ -785,9 +849,9 @@ void *cdw_list_display_get_current_item_data(CDW_LIST_DISPLAY *display)
    \return true if \p data is on a list associated with \p display
    \return false if \p data isn't on a list associated with \p display
 */
-bool cdw_list_display_is_member(CDW_LIST_DISPLAY *display, void *data, bool (*pred)(const void *, const void *))
+bool cdw_list_display_is_member(CDW_LIST_DISPLAY *display, void const * data, bool (*pred)(void const *, void const *))
 {
 	cdw_assert (display, "ERROR: display is NULL\n");
-	cdw_assert (pred != (bool (*)(const void *, const void *)) NULL, "ERROR: predicate function is NULL\n");
+	cdw_assert (pred != (bool (*)(void const *, void const *)) NULL, "ERROR: predicate function is NULL\n");
 	return cdw_dll_is_member(display->list, data, pred);
 }
