@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -128,12 +128,12 @@ cdw_rv_t cdw_main_window_module_init(void)
 	}
 
 	int rv = wrefresh(cdw_ui.window);
-	cdw_vdm_n ("wrefresh", rv, "");
+	cdw_vdm_n2 ("wrefresh", rv, "");
 
 	rv = keypad(cdw_ui.window, TRUE);
-	cdw_vdm_n ("keypad", rv, "");
+	cdw_vdm_n2 ("keypad", rv, "");
 	rv = wbkgd(cdw_ui.window, A_NORMAL | COLOR_PAIR(CDW_COLORS_MAIN));
-	cdw_vdm_n ("wbkgd", rv, "");
+	cdw_vdm_n2 ("wbkgd", rv, "");
 
 	/* left-hand, main app menu */
 	cdw_rv_t crv = cdw_main_window_main_menu_view_create();
@@ -144,18 +144,18 @@ cdw_rv_t cdw_main_window_module_init(void)
 	}
 
 	rv = mvwaddch(cdw_ui.window, 0, 1, ACS_RTEE);
-	cdw_vdm_n ("mvwaddch", rv, "");
+	cdw_vdm_n2 ("mvwaddch", rv, "");
 	rv = wattrset(cdw_ui.window, A_BOLD | COLOR_PAIR(CDW_COLORS_TITLE));
-	cdw_vdm_n ("wattrset", rv, "");
+	cdw_vdm_n2 ("wattrset", rv, "");
 	rv = mvwprintw(cdw_ui.window, 0, 2, " ::: %s %s ::: ", PACKAGE, VERSION);
-	cdw_vdm_n ("mvwprintw", rv, "");
+	cdw_vdm_n2 ("mvwprintw", rv, "");
 	rv = wattrset(cdw_ui.window, A_NORMAL | COLOR_PAIR(CDW_COLORS_MAIN));
-	cdw_vdm_n ("wattrset", rv, "");
+	cdw_vdm_n2 ("wattrset", rv, "");
 	rv = waddch(cdw_ui.window, ACS_LTEE);
-	cdw_vdm_n ("waddch", rv, "");
+	cdw_vdm_n2 ("waddch", rv, "");
 
 	rv = wrefresh(cdw_ui.window);
-	cdw_vdm_n ("wrefresh", rv, "");
+	cdw_vdm_n2 ("wrefresh", rv, "");
 
 	/* window in which selected files are displayed */
 	crv = cdw_main_window_files_list_view_create();
@@ -234,7 +234,7 @@ void cdw_main_window_module_clean(void)
 	   2. if you run cdw in 81x26 terminal */
 
 
-	cdw_file_manager_delete_selected_files_view();
+	cdw_selected_files_delete_view();
 	cdw_window_delete(&cdw_ui.files_list_view.subwindow);
 
 	cdw_window_delete(&cdw_ui.main_menu.menu_subwindow);
@@ -293,13 +293,11 @@ void cdw_main_window_recalculate_sizes(void)
 	/* -2: don't let letters in tooltips area be displayed very
 	   close to window borders */
 	cdw_ui.tooltips_view.n_cols = cdw_ui.n_cols - 2;
-	cdw_ui.tooltips_view.n_rows = 1;
+	cdw_ui.tooltips_view.n_rows = CDW_UI_TOOLTIPS_VIEW_N_ROWS;
 	cdw_ui.tooltips_view.begin_y = cdw_ui.n_rows - cdw_ui.tooltips_view.n_rows;
 	cdw_ui.tooltips_view.begin_x = 1;
 
-	/* disc info area needs 4 lines of usable space
-	   (Type?, Empty?, Writable?, Erasable?) + 2 lines for borders */
-        int lower_n_rows = 4 + 2;
+        int lower_n_rows = CDW_UI_VOLUME_VIEW_N_ROWS;
 	int upper_n_rows = cdw_ui.n_rows - lower_n_rows - cdw_ui.tooltips_view.n_rows;
 	int left_n_cols = MENU_LABEL_LEN;
 	int right_n_cols = cdw_ui.n_cols - left_n_cols;
@@ -496,7 +494,7 @@ cdw_rv_t cdw_main_window_files_list_view_create(void)
 			       _("Selected files"), (char *) NULL);
 	wattrset(cdw_ui.files_list_view.subwindow, A_NORMAL | COLOR_PAIR(CDW_COLORS_MAIN));
 
-	cdw_rv_t crv = cdw_file_manager_create_selected_files_view(cdw_ui.files_list_view.subwindow);
+	cdw_rv_t crv = cdw_selected_files_create_view(cdw_ui.files_list_view.subwindow);
 	if (crv == CDW_OK) {
 		return CDW_OK;
 	} else {
@@ -740,7 +738,7 @@ cdw_rv_t cdw_main_window_volume_info_view_update(long long disc_sectors_used, lo
 		}
 	}
 
-	size_t n_files = cdw_file_manager_number_of_selected_files();
+	size_t n_files = cdw_selected_files_get_number();
 	cdw_sdm ("INFO: number of selected files = %zd\n", n_files);
 
 	bool error = false;
@@ -748,7 +746,7 @@ cdw_rv_t cdw_main_window_volume_info_view_update(long long disc_sectors_used, lo
 	double selected_files_size_mb = 0.0;
 	if (n_files != 0) {
 		if (rescan_selected_files) {
-			selected_files_size_mb = cdw_file_manager_calculate_selected_files_size_mb();
+			selected_files_size_mb = cdw_selected_files_calculate_size_mb();
 			if (selected_files_size_mb < 0.0) {
 				error = true;
 			}
@@ -957,7 +955,7 @@ void cdw_main_window_volume_info_view_display_data_new(bool show_disc_info, size
 */
 cdw_rv_t cdw_main_window_delete_from_selected_files(void)
 {
-        size_t num = cdw_file_manager_number_of_selected_files();
+        size_t num = cdw_selected_files_get_number();
 
 	if (num > 0) { /* if cd files list not empty */
 		/* first set_menu_fore() is used to un-highlight main menu
@@ -1184,9 +1182,9 @@ void cdw_main_window_handle_follow_symlinks_change(bool old_state, bool current_
 		   and this may affect size of selected files */
 		cdw_vdm ("INFO: changed \"follow symlinks\" to %s, recalculating files size\n",
 			 current_state ? "true" : "false");
-		cdw_file_manager_follow_symlinks(current_state);
+		cdw_selected_files_set_follow_symlinks(current_state);
 		cdw_main_window_volume_info_view_update(-1, -1, true);
-		cdw_file_manager_regenerate_selected_files_view();
+		cdw_selected_files_regenerate_view();
 	} else {
 		cdw_vdm ("INFO: \"follow symlinks\" is still %s\n", current_state ? "true" : "false");
 	}
@@ -1201,10 +1199,10 @@ void cdw_about(void)
 {
 	cdw_buttons_dialog(_("About cdw"),
 			   /* 2TRANS: this is message in dialog window */
-			   _("cdw 0.8.0 - front end for tools creating ISO9660 and UDF file systems and writing data to optical discs.\n"
+			   _("cdw 0.8.1 - front end for tools creating ISO9660 and UDF file systems and writing data to optical discs.\n"
 
 			     "Copyright (C) 2002 Varkonyi Balazs\n"
-			     "Copyright (C) 2007 - 2014 Kamil Ignacak\n"
+			     "Copyright (C) 2007 - 2016 Kamil Ignacak\n"
 
 			     "This program is free software; you can redistribute it and/or modify\n"
 			     "it under the terms of the GNU General Public License as published by\n"
