@@ -52,6 +52,16 @@ typedef enum menu_items {
 } cdw_menu_items_t;
 
 
+/* Volume info area has 4 rows for contents + 2 rows for upper and
+   lower border. */
+/* Disc info area has the same height: 4 lines of usable space
+   (Type?, Empty?, Writable?, Erasable?) + 2 lines for borders */
+#define CDW_UI_VOLUME_VIEW_N_ROWS (4 + 2)
+
+/* The very last row in main window. Just one line of text. */
+#define CDW_UI_TOOLTIPS_VIEW_N_ROWS (1)
+
+
 cdw_rv_t cdw_main_window_module_init(void);
 void     cdw_main_window_module_clean(void);
 
