@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -111,15 +111,15 @@ cdw_rv_t cdw_ncurses_init(void)
 {
 	initscr();
 	int rv = start_color();
-	cdw_vdm_n ("start_color", rv, "");
+	cdw_vdm_n2 ("start_color", rv, "");
 	rv = cbreak();
-	cdw_vdm_n ("cbreak", rv, "");
+	cdw_vdm_n2 ("cbreak", rv, "");
 	rv = noecho();
-	cdw_vdm_n ("noecho", rv, "");
+	cdw_vdm_n2 ("noecho", rv, "");
 	rv = keypad(stdscr, TRUE);
-	cdw_vdm_n ("keypad", rv, "");
+	cdw_vdm_n2 ("keypad", rv, "");
 	rv = refresh();
-	cdw_vdm_n ("refresh", rv, "");
+	cdw_vdm_n2 ("refresh", rv, "");
 
 	if (cdw_commandline_arguments.escdelay_ms != -1) {
 		set_escdelay(cdw_commandline_arguments.escdelay_ms);
@@ -331,19 +331,19 @@ void cdw_ncurses_delete_menu_and_items(MENU **menu)
 		   # Free the items using free_item(). */
 
 		int rv = unpost_menu(*menu);
-		cdw_vdm_n ("unpost_menu", rv, "");
+		cdw_vdm_n2 ("unpost_menu", rv, "");
 
 		ITEM **items = menu_items(*menu);
 
 		rv = free_menu(*menu);
 		*menu = (MENU *) NULL;
-		cdw_vdm_n ("free_menu", rv, "");
+		cdw_vdm_n2 ("free_menu", rv, "");
 
 		if (items != (ITEM **) NULL) {
 			for (int i = 0; items[i] != (ITEM *) NULL; i++){
 				rv = free_item(items[i]);
 				items[i] = (ITEM *) NULL;
-				cdw_vdm_n ("free_item", rv, "");
+				cdw_vdm_n2 ("free_item", rv, "");
 			}
 
 			free(items);
@@ -377,6 +377,7 @@ static cdw_id_clabel_t cdw_ncurses_key_labels[] =
 		{ '\t',         "TAB" },
 		{ CDW_KEY_TAB,  "TAB" },
 		{ KEY_BTAB,     "BackTAB" },
+		{ KEY_BACKSPACE, "BackSpace" },
 		{ KEY_F(1),     "F1" },
 		{ KEY_F(2),     "F2" },
 		{ KEY_F(3),     "F3" },
@@ -397,7 +398,7 @@ static char cdw_ncurses_key_printable[2];
 
 const char *cdw_ncurses_key_label(int key)
 {
-	if (isprint(key)) {
+	if (isprint(key) && key != CDW_KEY_SPACE) { /* 'key != CDW_KEY_SPACE': don't print Space key as white char, print it as "SPACE" string instead. */
 		snprintf(cdw_ncurses_key_printable, 1 + 1, "%c", toascii(key));
 		return cdw_ncurses_key_printable;
 	} else {
@@ -750,5 +751,63 @@ char *cdw_ncurses_get_field_string(FIELD *field)
 		cdw_vdm ("INFO: returning field string \"%s\"\n", s);
 		return cdw_string_rtrim(s);
 	}
+}
 
+
+
+
+
+/**
+   \brief Get current position in form's field
+
+   Function returns position of a cursor in current field of given \p form.
+
+   \param form
+
+   \return position of cursor in current field
+*/
+int cdw_ncurses_get_field_cursor_pos(FORM *form)
+{
+	int n_chars = 0;
+
+	/* Find cursor's initial position by moving it back N times. */
+	while (E_OK == form_driver(form, REQ_PREV_CHAR)) {
+		n_chars++;
+	}
+
+	/* Restore cursor's position. */
+	for (int i = 0; i < n_chars; i++) {
+		if (E_OK != form_driver(form, REQ_NEXT_CHAR)) {
+			break;
+		}
+	}
+
+	return n_chars;
+}
+
+
+
+
+
+/**
+   \brief Move cursor to specified position in current field
+
+   Move cursor in current field of given \p form to specific position
+   \p cursor_pos.
+
+   \return CDW_OK on success
+   \return CDW_ERROR otherwise
+*/
+cdw_rv_t cdw_ncurses_set_field_cursor_pos(FORM *form, int cursor_pos)
+{
+	cdw_assert (cursor_pos >= 0, "ERROR: requested cursor is negative: %d\n", cursor_pos);
+
+	form_driver(form, REQ_BEG_LINE);
+	for (int i = 0; i < cursor_pos; i++) {
+		if (E_OK != form_driver(form, REQ_NEXT_CHAR)) {
+			return CDW_ERROR;
+		}
+	}
+
+	return CDW_OK;
 }
