@@ -8,7 +8,10 @@
 #include "cdw_ncurses.h"
 
 /* size of processwin */
-#define PROCESSWIN_COLS 40
+/* For large sizes of processed files the progress indication string
+   can be really long, so 50 cols, while large, seems to be
+   reasonable. */
+#define PROCESSWIN_COLS 50
 #define PROCESSWIN_ROWS 14
 /* maximal length of text that can be displayed in
    one line of processwin (does not include ending '\0') */
@@ -29,8 +32,12 @@ void cdw_processwin_display_fifo_and_speed(int fifo_level, int speed_decimal, in
 void cdw_processwin_erase_fifo_and_speed(void);
 void cdw_processwin_display_eta(const char *eta_line);
 void cdw_processwin_erase_eta(void);
-void cdw_processwin_display_progress_conditional(long current, long total, const char *description);
+
+void cdw_processwin_display_percentage_progress(double percent_done, const char *description);
+void cdw_processwin_display_percentage_progress_conditional(double current_percent, const char *description);
 void cdw_processwin_display_progress(long current_amount, long total_amount, const char *description);
+void cdw_processwin_display_progress_conditional(long current, long total, const char *description);
+
 void cdw_processwin_add_progress_bar(void);
 void cdw_processwin_delete_progress_bar(void);
 
@@ -40,4 +47,3 @@ void cdw_processwin_reset_progress(void);
 void cdw_processwin_force_refresh(void);
 
 #endif /* H_CDW_PROCESSWIN */
-
