@@ -1,5 +1,5 @@
 #ifndef H_CDW_VERIFY_WIZARD
-#define H_CDWE_VERIFY_WIZARD
+#define H_CDW_VERIFY_WIZARD
 
 #include "main.h"
 
