@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,13 +64,17 @@ WINDOW *cdw_window_new(WINDOW *parent, int n_lines, int n_cols, int begin_y, int
 	if (!parent) {
 		window = newwin(n_lines, n_cols, begin_y, begin_x);
 	} else {
+		cdw_assert (begin_x < getmaxx(parent), "ERROR: begin_x (%d) >= getmaxx(parent) (%d)\n", begin_x, getmaxx(parent));
+		cdw_assert (begin_y < getmaxy(parent), "ERROR: begin_y (%d) >= getmaxy(parent) (%d)\n", begin_y, getmaxy(parent));
+		cdw_vdm ("INFO: creating derived window from window of size y = %d / x = %d\n", getmaxy(parent), getmaxx(parent));
 		window = derwin(parent, n_lines, n_cols, begin_y, begin_x);
 	}
 	if (!window) {
 		cdw_vdm ("ERROR: failed to create %s window\n",
 			 parent == (WINDOW *) NULL ? "new" : "derived");
-		cdw_vdm ("ERROR: problem occurred when creating \"%s\" / \"%s\" window\n",
-			 top_string, bottom_string);
+		cdw_vdm ("ERROR: problem occurred when creating \"%s\" / \"%s\" window: n_lines = %d, n_cols = %d, begin_y = %d, begin_x = %d\n",
+			 top_string, bottom_string,
+			 n_lines, n_cols, begin_y, begin_x);
 		return (WINDOW *) NULL;
 	}
 
@@ -140,7 +144,7 @@ void cdw_window_delete(WINDOW **window)
 	} else {
 		int rv = delwin(*window);
 		*window = (WINDOW *) NULL;
-		cdw_vdm_n ("delwin", rv, "");
+		cdw_vdm_n2 ("delwin", rv, "");
 		return;
 	}
 }
@@ -308,7 +312,7 @@ int cdw_window_print_message(WINDOW *window, const char *message, int align)
 
 	int n_cols = getmaxx(window);
 	int n_rows = getmaxy(window);
-	int n_lines = (int) cdw_string_count_lines(message, (size_t) n_cols);
+	int n_lines = (int) cdw_string_count_lines_new(message, (size_t) n_cols);
 	if (n_lines > n_rows) {
 		cdw_vdm ("WARNING: message has more lines (%d) than number of rows in the window (%d)\n",
 			 n_lines, n_rows);
