@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -290,19 +290,19 @@ cdw_rv_t cdw_write_wizard_init(cdw_task_t *task, cdw_disc_t *disc)
 
 int cdw_write_wizard_driver(void)
 {
-	int fi = f_cancel_b; /* Initial focus on "Cancel" button. */
 	int key = 'a';
+	int fi = f_cancel_b; /* Initial focus on "Cancel" button. */
+	cdw_form_driver_focus_on(wizard.cdw_form, fi);
 
 	while (key != CDW_KEY_ESCAPE && key != 'q' && key != 'Q') {
-		key = cdw_form_driver(wizard.cdw_form, fi);
+		key = cdw_form_driver_new(wizard.cdw_form, &fi);
 
 		/* In top-level driver of wizard we are mainly
-		   interested in ENTER being pressed on either "write"
-		   or "cancel" button.  If user presses ENTER on
+		   interested in ENTER being pressed on either "Write"
+		   or "Cancel" button.  If user presses ENTER on
 		   "Cancel" button, form driver will return ESCAPE
 		   key. */
 		if (key == CDW_KEY_ENTER) {
-			fi = field_index(current_field(wizard.cdw_form->form));
 			if (fi != f_write_b) {
 				/* We are not on "Write" button, so do nothing, wait for next key. */
 				continue;
