@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,14 +46,14 @@
 /**
    \brief Create new CDW_BUTTON widget
 
-   \date Function's top-level comment reviewed on 2012-01-09
-   \date Function's body reviewed on 2012-01-09
+   \date Function's top-level comment reviewed on 2016-02-16
+   \date Function's body reviewed on 2016-02-16
 
    Remember to assign value to CDW_BUTTON->on_click_callback
 
    \param parent - parent window, in which the button will be displayed
-   \param begin_y - number of row of parent window, in which will be displayed upper border of button
-   \param begin_x - number of column of parent window, in which will be displayed left border of button
+   \param begin_y - number of row of parent window, in which the button will be placed
+   \param begin_x - number of column of parent window, in which the button will be placed
    \param label - string that will be displayed as button label
    \param colors - color scheme of button
 
@@ -63,6 +63,12 @@
 CDW_BUTTON *cdw_button_new(WINDOW *parent, int begin_y, int begin_x, const char *label, cdw_colors_t colors)
 {
 	cdw_assert (parent, "ERROR: parent window is null\n");
+	/* Don't let this widget begin outside of its parent
+	   window. Button is one-line widget, so we don't have to
+	   check its lower border - conditions are the same as for
+	   upper border.. */
+	cdw_assert (begin_x < getmaxx(parent), "ERROR: begin_x (%d) >= getmaxx(parent) (%d)\n", begin_x, getmaxx(parent));
+	cdw_assert (begin_y < getmaxy(parent), "ERROR: begin_y (%d) >= getmaxy(parent) (%d)\n", begin_y, getmaxy(parent));
 
 	CDW_BUTTON *button = (CDW_BUTTON *) malloc(sizeof(CDW_BUTTON));
 	if (!button) {
@@ -70,9 +76,13 @@ CDW_BUTTON *cdw_button_new(WINDOW *parent, int begin_y, int begin_x, const char
 		return (CDW_BUTTON *) NULL;
 	}
 
+	cdw_widget_init(&button->widget);
+	cdw_widget_add_return_keys(&button->widget, CDW_KEY_ESCAPE, CDW_KEY_TAB, CDW_KEY_BTAB, KEY_UP, KEY_DOWN, 0);
+	button->widget.add_return_key = cdw_widget_add_return_key; /* Default function for adding return keys by parent cdw_form. */
+	button->widget.type_id = CDW_WIDGET_ID_BUTTON;
+
 	button->strip = (WINDOW *) NULL;
 	button->label = (char *) NULL;
-	button->widget_id = CDW_WIDGET_ID_BUTTON;
 	button->on_click_key = 0;
 
 	button->label = strdup(label);
@@ -96,8 +106,8 @@ CDW_BUTTON *cdw_button_new(WINDOW *parent, int begin_y, int begin_x, const char
 			       button->begin_y, button->begin_x);
 	if (!button->strip) {
 		cdw_vdm ("ERROR: can't create new button: derwin()\n");
-		cdw_vdm ("ERROR: nlines = %d, ncols = %d, begin_y = %d, begin_x = %d\n",
-			 1, 2 + label_len + 2, button->begin_y, button->begin_x);
+		cdw_vdm ("ERROR: nlines = %d, ncols = %d, begin_y = %d, begin_x = %d, getmaxx(parent) = %d, getmaxy(parent) = %d\n",
+			 1, 2 + label_len + 2, button->begin_y, button->begin_x, getmaxx(parent), getmaxy(parent));
 
 		cdw_button_delete(&button);
 		return (CDW_BUTTON *) NULL;
@@ -240,11 +250,11 @@ void cdw_button_unfocus(CDW_BUTTON *button)
 int cdw_button_driver(CDW_BUTTON *b, void *form)
 {
 	cdw_assert (b, "ERROR: NULL pointer to button\n");
-	cdw_assert (b->widget_id == CDW_WIDGET_ID_BUTTON, "ERROR: this is not a button widget\n");
+	cdw_assert (b->widget.type_id == CDW_WIDGET_ID_BUTTON, "ERROR: this is not a button widget\n");
 	cdw_assert (b->parent, "ERROR: can't control button without parent window\n");
 
 	int key = 'a'; /* Safe initial value. */
-	while (1) {
+	while (!cdw_widget_is_return_key(&b->widget, key)) {
 		key = wgetch(b->parent);
 
 		if (key == CDW_KEY_ENTER) {
@@ -257,24 +267,18 @@ int cdw_button_driver(CDW_BUTTON *b, void *form)
 				cdw_vdm ("INFO: Returning key %s\n", cdw_ncurses_key_label(b->on_click_key));
 				return b->on_click_key;
 			}
-		} else if (key == CDW_KEY_TAB || key == KEY_BTAB
-			   || key == KEY_DOWN || key == KEY_UP) {
-
-			/* Inter-widget movement key. Will be handled
-			   by parent of the widget. */
-			break;
-		} else if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
-
-			key = CDW_KEY_ESCAPE;
-			break;
 		} else {
-			/* Some other key that may be handled only by
-			   parent of the widget. It may happen that
-			   parent is configured to handle special keys
-			   like F2 or other keys. */
-			break;
+			/* Either widget's return key that will be
+			   caught by loop condition and handled by
+			   widget's parent, or a completely
+			   meaningless key that should be ignored. */
+			;
 		}
 	}
 
+	if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
+		key = CDW_KEY_ESCAPE;
+	}
+
 	return key;
 }
