@@ -7,6 +7,9 @@
 
 
 typedef struct {
+
+	cdw_widget_t widget;
+
 	WINDOW *parent; /* parent window of the widget */
 	WINDOW *strip;  /* a private window in which the widget displays its content */
 
@@ -20,10 +23,6 @@ typedef struct {
 
 	cdw_colors_t colors;
 
-	/* The variable will be set to CDW_WIDGET_ID_BUTTON by
-	   cdw_button_new(). */
-	int widget_id;
-
 } CDW_BUTTON;
 
 
