@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -82,13 +82,16 @@ CDW_CHECKBOX *cdw_checkbox_new(WINDOW *parent, int begin_y, int begin_x, bool ch
 		return (CDW_CHECKBOX *) NULL;
 	}
 
+	cdw_widget_init(&checkbox->widget);
+	cdw_widget_add_return_keys(&checkbox->widget, CDW_KEY_ESCAPE, CDW_KEY_TAB, CDW_KEY_BTAB, KEY_UP, KEY_DOWN, 0);
+	checkbox->widget.add_return_key = cdw_widget_add_return_key; /* Default function for adding return keys by parent cdw_form. */
+	checkbox->widget.type_id = CDW_WIDGET_ID_CHECKBOX;
+
 	checkbox->parent = parent;
 	checkbox->begin_y = begin_y;
 	checkbox->begin_x = begin_x;
 	checkbox->checked = checked;
 
-	checkbox->widget_id = CDW_WIDGET_ID_CHECKBOX;
-
 	checkbox->visible = true;
 	checkbox->on_toggle_callback = (cdw_form_widget_function_t) NULL;
 
@@ -328,40 +331,33 @@ void cdw_checkbox_draw(CDW_CHECKBOX *checkbox)
 int cdw_checkbox_driver(CDW_CHECKBOX *cb, void *cdw_form)
 {
 	cdw_assert (cb, "ERROR: NULL pointer to checkbox\n");
-	cdw_assert (cb->widget_id == CDW_WIDGET_ID_CHECKBOX, "ERROR: this is not a checkbox widget\n");
+	cdw_assert (cb->widget.type_id == CDW_WIDGET_ID_CHECKBOX, "ERROR: this is not a checkbox widget\n");
 	cdw_assert (cb->parent, "ERROR: cannot control checkbox with NULL parent\n");
 
 	int key = 'a'; /* Safe initial value. */
-	while (1) {
+	while (!cdw_widget_is_return_key(&cb->widget, key)) {
 		key = wgetch(cb->parent);
 
 		if (key == ' ' || key == 'x' || key == 'X') {
-
 			cdw_checkbox_toggle(cb);
 			if (cb->on_toggle_callback) {
 				cdw_assert (cdw_form, "ERROR: 'cdw_form' argument to callback is NULL\n");
 				bool state = cdw_checkbox_get_state(cb);
 				cb->on_toggle_callback((cdw_form_t *) cdw_form, &state);
 			}
-		} else if (key == CDW_KEY_TAB || key == KEY_BTAB
-			   || key == KEY_DOWN || key == KEY_UP) {
-
-			/* Inter-widget movement key. Will be handled
-			   by parent of the widget. */
-			break;
-		} else if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
-
-			key = CDW_KEY_ESCAPE;
-			break;
 		} else {
-			/* Some other key that may be handled only by
-			   parent of the widget. It may happen that
-			   parent is configured to handle special keys
-			   like F2 or other keys. */
-			break;
+			/* Either widget's return key that will be
+			   caught by loop condition and handled by
+			   widget's parent, or a completely
+			   meaningless key that should be ignored. */
+			;
 		}
 	}
 
+	if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
+		key = CDW_KEY_ESCAPE;
+	}
+
 	redrawwin(cb->parent);
 	wrefresh(cb->parent);
 
