@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -515,7 +515,7 @@ cdw_rv_t cdw_dialog_base_txt_sizes(CDW_DIALOG_BASE *dialog, const char *message)
 */
 int cdw_dialog_base_txt_sizes_rows(CDW_DIALOG_BASE *dialog, const char *message)
 {
-	dialog->txt.n_rows = (int) cdw_string_count_lines(message, (size_t) dialog->txt.n_cols);
+	dialog->txt.n_rows = (int) cdw_string_count_lines_new(message, (size_t) dialog->txt.n_cols);
 	int n_rows = dialog->txt.n_rows + 2 * dialog->pad_y + dialog->input.spacer_y + dialog->input.n_rows;
 
 	return n_rows;
@@ -1001,12 +1001,12 @@ void cdw_input_dialog_delete(CDW_INPUT_DIALOG **d)
 
 int cdw_input_dialog_driver(CDW_INPUT_DIALOG *d)
 {
-	int fi = f_input_i; /* Initial focus on input. */
 	int key = 'a';      /* Safe initial value. */
+	int fi = f_input_i; /* Initial focus on input field. */
+	cdw_form_driver_focus_on(d->cdw_form, fi);
 
 	while (key != CDW_KEY_ESCAPE && key != 'q' && key != 'Q') {
-		key = cdw_form_driver(d->cdw_form, fi);
-		fi = field_index(current_field(d->cdw_form->form));
+		key = cdw_form_driver_new(d->cdw_form, &fi);
 
 		if (key == CDW_KEY_ENTER) {
 			cdw_assert (fi == f_ok_b, "ERROR: driver returns KEY_ENTER for widget other than OK button (%d)\n", fi);
@@ -1180,7 +1180,7 @@ cdw_rv_t cdw_input_dialog_sizes(CDW_INPUT_DIALOG *d)
 	d->n_cols = 50;
 	d->sub_n_cols = d->n_cols - 2 * cols_space;
 
-	d->txt_n_rows = (int) cdw_string_count_lines(d->prompt_message, (size_t) d->sub_n_cols);
+	d->txt_n_rows = (int) cdw_string_count_lines_new(d->prompt_message, (size_t) d->sub_n_cols);
 
 	d->n_rows = 2 * rows_space        /* Parent window should be larger than subwindow by X rows on top and on bottom. */
 		+ d->txt_n_rows           /* Prompt on top. */
