@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -80,13 +80,16 @@ CDW_DROPDOWN *cdw_dropdown_new(WINDOW *parent, int begin_y, int begin_x, int wid
 		return (CDW_DROPDOWN *) NULL;
 	}
 
+	cdw_widget_init(&dropdown->widget);
+	cdw_widget_add_return_keys(&dropdown->widget, CDW_KEY_ESCAPE, CDW_KEY_TAB, CDW_KEY_BTAB, KEY_UP, KEY_DOWN, 0);
+	dropdown->widget.add_return_key = cdw_widget_add_return_key; /* Default function for adding return keys by parent cdw_form. */
+	dropdown->widget.type_id = CDW_WIDGET_ID_DROPDOWN;
+
 	dropdown->n_items_max = n_items_max;
 	dropdown->n_items = 0;
 
 	dropdown->current_item_ind = 0; /* There will be at least one item in the dropdown, so index=0 should be safe. */
 
-	dropdown->widget_id = CDW_WIDGET_ID_DROPDOWN;
-
 	dropdown->items = (cdw_id_label_t **) malloc((size_t) (dropdown->n_items_max + 1) * sizeof (cdw_id_label_t *));
 
 	if (!dropdown->items) {
@@ -401,12 +404,12 @@ void cdw_dropdown_make_visible(CDW_DROPDOWN *dropdown)
 int cdw_dropdown_driver(CDW_DROPDOWN *dd, __attribute__((unused)) void *dummy)
 {
 	cdw_assert (dd, "ERROR: NULL pointer to dropdown\n");
-	cdw_assert (dd->widget_id == CDW_WIDGET_ID_DROPDOWN, "ERROR: this is not a dropdown widget\n");
+	cdw_assert (dd->widget.type_id == CDW_WIDGET_ID_DROPDOWN, "ERROR: this is not a dropdown widget\n");
 	cdw_assert (dd->parent, "ERROR: cannot control dropdown with NULL parent\n");
 
 	int key = 'a'; /* Safe initial value. */
 
-	while (1) {
+	while (!cdw_widget_is_return_key(&dd->widget, key)) {
 		key = wgetch(dd->parent);
 
 		if (key == CDW_KEY_ENTER) {
@@ -425,25 +428,19 @@ int cdw_dropdown_driver(CDW_DROPDOWN *dd, __attribute__((unused)) void *dummy)
 					dd->on_select_callback(NULL, &(dd->items[dd->current_item_ind]->id));
 				}
 			}
-		} else if (key == CDW_KEY_TAB || key == KEY_BTAB
-			   || key == KEY_DOWN || key == KEY_UP) {
-
-			/* Inter-widget movement key. Will be handled
-			   by parent of the widget. */
-			break;
-		} else if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
-
-			key = CDW_KEY_ESCAPE;
-			break;
 		} else {
-			/* Some other key that may be handled only by
-			   parent of the widget. It may happen that
-			   parent is configured to handle special keys
-			   like F2 or other keys. */
-			break;
+			/* Either widget's return key that will be
+			   caught by loop condition and handled by
+			   widget's parent, or a completely
+			   meaningless key that should be ignored. */
+			;
 		}
 	}
 
+	if (key == CDW_KEY_ESCAPE || key == 'q' || key == 'Q') {
+		key = CDW_KEY_ESCAPE;
+	}
+
 	redrawwin(dd->parent);
 	wrefresh(dd->parent);
 
