@@ -2,11 +2,15 @@
 #define H_CDW_DROPDOWN
 
 
+#include "main.h"
 #include "cdw_ncurses.h"
 
 
 
 typedef struct {
+
+	cdw_widget_t widget;
+
 	WINDOW *parent; /* parent window, in which a dropdown will be displayed */
 	int begin_x;    /* x-coordinate of a dropdown in parent window */
 	int begin_y;    /* y-coordinate of a dropdown in parent window */
@@ -30,10 +34,6 @@ typedef struct {
 	bool visible;            /* is widget visible and visitable? */
 	cdw_id_label_t **items;  /* the content of the widget */
 
-	/* The variable will be set to CDW_WIDGET_ID_DROPDOWN by
-	   cdw_dropdown_new(). */
-	int widget_id;
-
 	/* internal details of implementation */
 	MENU   *menu;
 	ITEM  **menu_items;
