@@ -6,15 +6,15 @@
 
 
 typedef struct {
+
+	cdw_widget_t widget;
+
 	WINDOW *parent;  /* parent window of the widget */
 
 	char *content;
 	FIELD *field;
 	int n_cols;
 
-	/* For consistency with other widget types that have driver function. */
-	int widget_id;
-
 } CDW_DYNAMIC_LABEL;
 
 
