@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,6 +59,9 @@ static void     cdw_safe_input_line_print_message(CDW_SAFE_INPUT_LINE *line, cha
 static void     cdw_safe_input_line_prepare_message(char *message, int len_max, char *insecure);
 static int      cdw_safe_input_line_form_driver(CDW_SAFE_INPUT_LINE *line);
 static cdw_rv_t cdw_safe_input_line_test_content_security(CDW_SAFE_INPUT_LINE *line, char *insecure);
+static void     cdw_safe_input_line_add_return_key(cdw_widget_t *widget, int key);
+
+
 
 
 
@@ -110,6 +113,11 @@ CDW_SAFE_INPUT_LINE *cdw_safe_input_line_new(WINDOW *parent, FORM **form, FIELD
 		return (CDW_SAFE_INPUT_LINE *) NULL;
 	}
 
+	cdw_widget_init(&line->widget);
+	cdw_widget_add_return_keys(&line->widget, CDW_KEY_ESCAPE, CDW_KEY_TAB, CDW_KEY_BTAB, KEY_UP, KEY_DOWN, 0);
+	line->widget.add_return_key = cdw_safe_input_line_add_return_key; /* Widget's custom function for adding return keys by parent cdw_form. */
+	line->widget.type_id = CDW_WIDGET_ID_SAFE_INPUT_LINE;
+
 	line->parent = parent;
 	line->form = form;
 	line->field = field;
@@ -129,10 +137,10 @@ CDW_SAFE_INPUT_LINE *cdw_safe_input_line_new(WINDOW *parent, FORM **form, FIELD
 		/* TODO: shouldn't we call set_field_buffer(line->field, 0, "") ? */
 	}
 
-	line->input_type = input_type;
 	line->chars_max = chars_max;
+	line->input_type = input_type;
+	line->cursor_pos = 0;
 
-	line->widget_id = CDW_WIDGET_ID_SAFE_INPUT_LINE;
 	line->attempts_max = CDW_SAFE_INPUT_LINE_ATTEMPTS_MAX;
 
 	line->message_area = (CDW_DYNAMIC_LABEL *) NULL;
@@ -208,14 +216,15 @@ void cdw_safe_input_line_bind_message_area(CDW_SAFE_INPUT_LINE *line, CDW_DYNAMI
 /**
    \brief Top-level driver for input line widget
 
-   \date Function's top-level comment reviewed on 2014-03-30
-   \date Function's body reviewed on 2014-03-30
+   \date Function's top-level comment reviewed on 2016-02-07
+   \date Function's body reviewed on 2016-02-07
 
-   Driver for input line widget - it handles keyboard input when
+   Driver for safe input line widget - it handles keyboard input when
    keyboard focus is on the widget.
 
-   Enter key is not treated as a special key - it is *not* treated as
-   "I'm telling you to accept this input" command from user.
+   Unless configured as return key, Enter key is not treated as a
+   special key - it is *not* treated as "I'm telling you to accept
+   this input" command from user.
 
    Upon detecting TAB, BackTAB, Down or Up key the function checks the
    string entered by user for insecure characters, and if there are
@@ -228,12 +237,13 @@ void cdw_safe_input_line_bind_message_area(CDW_SAFE_INPUT_LINE *line, CDW_DYNAMI
 
    \return CDW_KEY_TAB, KEY_BTAB, KEY_DOWN or KEY_UP if user pressed any of those keys
    \return CDW_KEY_ESCAPE if user pressed Escape key
+   \return CDW_KEY_ENTER if user pressed Enter key, and the key is configured as return key
    \return KEY_EXIT on errors
 */
 int cdw_safe_input_line_driver(CDW_SAFE_INPUT_LINE *line, __attribute__((unused)) void *dummy)
 {
 	cdw_assert (line, "ERROR: NULL pointer to safe input line\n");
-	cdw_assert (line->widget_id == CDW_WIDGET_ID_SAFE_INPUT_LINE, "ERROR: this is not a safe input line widget\n");
+	cdw_assert (line->widget.type_id == CDW_WIDGET_ID_SAFE_INPUT_LINE, "ERROR: this is not a safe input line widget\n");
 	cdw_assert (line->parent, "ERROR: can't control safe input line without parent window\n");
 
 	int key = 'a'; /* Safe initial value. */
@@ -244,9 +254,12 @@ int cdw_safe_input_line_driver(CDW_SAFE_INPUT_LINE *line, __attribute__((unused)
 		key = cdw_safe_input_line_form_driver(line);
 		curs_set(0);
 
-		if (key == CDW_KEY_TAB || key == KEY_BTAB
-		    || key == KEY_DOWN || key == KEY_UP) {
-
+		if (key == CDW_KEY_ESCAPE) {
+			/* This is also return key, but deserves
+			   special treatment (i.e. no action at all).
+			   Handle it before any other return key. */
+			break;
+		} else if (cdw_widget_is_return_key(&line->widget, key)) {
 			/* Don't allow leaving the field when there is
 			   unsafe input in it. */
 
@@ -277,8 +290,6 @@ int cdw_safe_input_line_driver(CDW_SAFE_INPUT_LINE *line, __attribute__((unused)
 			cdw_vdm ("ERROR: failed to get field buffer\n");
 			key = KEY_EXIT;
 			break;
-		} else if (key == CDW_KEY_ESCAPE) {
-			break;
 		} else {
 			/* May be a control key - to be investigated
 			   by parent. */
@@ -485,15 +496,15 @@ cdw_rv_t cdw_safe_input_line_set_content(CDW_SAFE_INPUT_LINE *line, const char *
 /**
    \brief Low-level input line driver
 
-   \date Function's top-level comment reviewed on 2014-03-30
-   \date Function's body reviewed on 2014-03-30
+   \date Function's top-level comment reviewed on 2016-02-07
+   \date Function's body reviewed on 2016-02-07
 
    Low level driver used by cdw_safe_input_line_driver(). It deals
    directly with ncurses form/field that is the heart of the safe
    input line widget.
 
-   Function returns when user presses TAB, BackTAB, Up, Down or
-   Escape.  The function does not return when user presses ENTER.
+   Function returns on configured return keys (TAB, BackTAB, Up, Down,
+   Escape. Also on Enter (if configured by client code)).
 
    Returned key should be interpreted by caller, and content of the
    widget should be extracted from widget and used as caller wants it.
@@ -503,27 +514,35 @@ cdw_rv_t cdw_safe_input_line_set_content(CDW_SAFE_INPUT_LINE *line, const char *
 
    \return CDW_KEY_ESCAPE if user pressed ESCAPE key.
    \return CDW_KEY_TAB, KEY_BTAB, KEY_DOWN or KEY_UP if user pressed any of those keys
+   \return CDW_KEY_ENTER if Enter is configured as return key
    \return KEY_EXIT on errors
 */
 int cdw_safe_input_line_form_driver(CDW_SAFE_INPUT_LINE *line)
 {
-	bool loop = true; /* Flag controlling outer do/while loop. */
-	int key = 0;
+	int key = 'a';
 
-	do {
+	cdw_ncurses_set_field_cursor_pos(*(line->form), line->cursor_pos);
+
+	while (!cdw_widget_is_return_key(&line->widget, key)) {
 		int rv = E_OK;
 		key = wgetch(line->parent);
 		switch (key) {
 			case KEY_LEFT:
 				rv = form_driver(*(line->form), REQ_PREV_CHAR);
+				if (rv == E_OK) {
+					line->cursor_pos--;
+				}
 				break;
 			case KEY_RIGHT:
 				rv = form_driver(*(line->form), REQ_NEXT_CHAR);
+				if (rv == E_OK) {
+					line->cursor_pos++;
+				}
 				break;
 			case KEY_BACKSPACE:
 				/* If cursor is at the beginning of
 				   input field, this can be
-				   interpreted a "move to previous
+				   interpreted as "move to previous
 				   field in a form". I don't want to
 				   move to previous field. */
 
@@ -539,6 +558,9 @@ int cdw_safe_input_line_form_driver(CDW_SAFE_INPUT_LINE *line)
 
 					/* Delete the char as requested. */
 					rv = form_driver(*(line->form), REQ_DEL_PREV);
+					if (rv == E_OK) {
+						line->cursor_pos--;
+					}
 					break;
 				}
 
@@ -547,50 +569,41 @@ int cdw_safe_input_line_form_driver(CDW_SAFE_INPUT_LINE *line)
 				break;
 		        case KEY_HOME:
 				rv = form_driver(*(line->form), REQ_BEG_LINE);
+				if (rv == E_OK) {
+					line->cursor_pos = 0;
+				}
 				break;
 		        case KEY_END:
 				rv = form_driver(*(line->form), REQ_END_LINE);
-				break;
-		        case CDW_KEY_ENTER:
-				/* Ignore the key. The only way to
-				   leave the field is by
-				   TAB/BackTAB/Down/Up/ESCAPE. The
-				   Enter key does not mean "OK, now
-				   close the window that uses this
-				   input line". */
-				break;
-			case CDW_KEY_ESCAPE:
-		        case CDW_KEY_TAB:
-		        case KEY_BTAB:
-		        case KEY_UP:
-		        case KEY_DOWN:
-				/* These keys are the only way to
-				   correctly make the keyboard focus
-				   move outside of the widget. */
-				loop = false;
+				if (rv == E_OK) {
+					line->cursor_pos = cdw_ncurses_get_field_cursor_pos(*(line->form));
+				}
 				break;
 			default:
 				if (isprint(key)) {
-					/* Push user input to form. */
+					/* Push entered text to form. */
 					rv = form_driver(*(line->form), key);
+					if (rv == E_OK) {
+						line->cursor_pos++;
+					}
 					wrefresh(line->parent);
-					break;
-				} else {
-					/* Probably a control key. Let parent decide. */
-					loop = false;
-					break;
 				}
+				break;
 		} /* switch () */
 
 		if (rv != E_OK) {
 			cdw_vdm ("WARNING: form_driver(..., key) returns %s for key %s\n",
 				 cdw_ncurses_error_string(rv), cdw_ncurses_key_label(key));
 		}
+	}
 
-	} while (loop);
+	if (key == CDW_KEY_ESCAPE) { /* This is the only "quit" key for this widget. 'Q'/'q' doesn't work. */
 
-	if (key == CDW_KEY_ESCAPE) {
-		/* No need to validate field content. */
+		/* No need to validate field content since Escape will
+		   discard the contents. Notice that not doing
+		   validation/flushing may result in not-up-to-data
+		   contents of field buffer. Remember about it when
+		   you decide to print the buffer on Escape. */
 		return key;
 	}
 
@@ -601,7 +614,7 @@ int cdw_safe_input_line_form_driver(CDW_SAFE_INPUT_LINE *line)
 			 cdw_ncurses_error_string(rv), cdw_ncurses_key_label(key));
 		return KEY_EXIT;
 	} else {
-		/* Escape, Tab, BTab, Down or Up. */
+		/* Escape, Tab, BTab, Down or Up (or perhaps Enter). */
 		return key;
 	}
 }
@@ -692,3 +705,25 @@ cdw_rv_t cdw_safe_input_line_test_content_security(CDW_SAFE_INPUT_LINE *line, ch
 	local_buffer = (char *) NULL;
 	return sec;
 }
+
+
+
+
+void cdw_safe_input_line_add_return_key(cdw_widget_t *widget, int key)
+{
+	cdw_assert (widget, "ERROR: \"widget\" argument is NULL\n");
+	cdw_assert (widget->n_return_keys < CDW_WIDGET_N_RETURN_KEYS_MAX,
+		    "ERROR: there are already %d / %d return keys in the widget, can't add another one\n",
+		    widget->n_return_keys, CDW_WIDGET_N_RETURN_KEYS_MAX);
+	cdw_assert (key != 0, "ERROR: trying to add key == 0, but 0 is an initializer value\n");
+
+	if (isprint(key)) {
+		/* Keys satisfying 'isprint()' should be entered into
+		   input line, so they can't be return keys. */
+		return;
+	}
+
+	cdw_widget_add_return_key(widget, key);
+
+	return;
+}
