@@ -25,7 +25,7 @@
  * \brief Print debug message - verbose version
  *
  * This macro behaves much like fprintf(stderr, ...) function,
- * caller only have to provide format string with converesion specifiers
+ * caller only have to provide format string with conversion specifiers
  * and list of arguments for this format string.
  *
  * Each message is preceeded with name of function that called the macro.
@@ -86,6 +86,44 @@
 
 
 
+/**
+  \brief Print debug message for ncurses return values - less verbose version
+
+  This macro behaves much like fprintf(stderr, ...) function,
+  caller only have to provide format string with converesion specifiers
+  and list of arguments for this format string.
+
+  If returned value is E_OK, nothing is printed.
+
+  Each message is preceeded with name of function that called the macro.
+
+  See "C: A Reference Manual", chapter 3.3.10 for more information
+  on variable argument lists in macros (it requires C99).
+*/
+#ifndef NDEBUG
+#define cdw_vdm_n2(m_fname, m_rv, ...)					\
+	{								\
+		int m_e = errno;					\
+		if (m_rv == E_OK) {					\
+			; /* Be silent about no-error situations */	\
+		} else {						\
+			fprintf(stderr, "%s():%d: ", __func__, __LINE__);	\
+			fprintf(stderr, "ERROR: %s() returns %s %s\n", m_fname, cdw_ncurses_error_string(rv), __VA_ARGS__); \
+			if (m_rv == E_SYSTEM_ERROR) {			\
+				fprintf(stderr, "%s():%d: ", __func__, __LINE__); \
+				fprintf(stderr, "ERROR: %s(): error = %s\n", m_fname, strerror(m_e)); \
+			}						\
+			fflush(stderr);					\
+		}							\
+	}
+#else
+#define cdw_vdm_n2(m_fname, m_rv, ...)
+#endif
+
+
+
+
+
 /** \brief Print debug message for ncurses return values - silent version */
 #define cdw_sdm_n(...)
 
