@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -206,8 +206,8 @@ cdw_rv_t cdw_dll_append(cdw_dll_item_t **head, void *data, bool (*pred)(const vo
  *
  * \param i - index of item that you want to obtain (0-based)
  *
- * \return Pointer to i-th item of list
- * \return NULL if index is too large or list is empty
+ * \return Pointer to i-th item of list on success
+ * \return NULL if given index is larger than length of given list, or if list is empty
  */
 cdw_dll_item_t *cdw_dll_ith_item(cdw_dll_item_t *head, size_t i)
 {
@@ -409,7 +409,7 @@ bool cdw_dll_is_empty(cdw_dll_item_t *head)
  * the file is on the list, return false if the file is not on
  * the list.
  *
- * The function also works for empty list: it returns false.
+ * The function also works for empty list (head == NULL): it returns false.
  *
  * This piece of code is used in only one place, but I put it in
  * function so that I can test it more easily. Additionally caller
@@ -418,19 +418,18 @@ bool cdw_dll_is_empty(cdw_dll_item_t *head)
  * \param cdw_file - file which you want to look for on the list
  *
  * \return true if given file is on the list
- * \return false if given file is not on the list (or the list is empty)
+ * \return false if given file is not on the list (or the list is empty, or the predicate function pointer is NULL)
  */
-bool cdw_dll_is_member(cdw_dll_item_t *head, void *data, bool (*pred)(const void *, const void *))
+bool cdw_dll_is_member(cdw_dll_item_t const * head, void const * data, bool (*pred)(void const *, void const *))
 {
-	cdw_assert (data != (void *) NULL, "data is null\n");
-	cdw_assert (pred != NULL, "predicate function is null\n");
+	cdw_assert (data, "data is NULL\n");
+	cdw_assert (pred, "predicate function is NULL\n");
 
-	if (head == (cdw_dll_item_t *) NULL) {
+	if (!data || !pred) {
 		return false;
 	}
 
-	cdw_dll_item_t *f;
-	for (f = head; f != (cdw_dll_item_t *) NULL; f = f->next) {
+	for (cdw_dll_item_t const * f = head; f; f = f->next) {
 		if (pred(data, f->data)) {
 			return true;
 		}
