@@ -24,7 +24,7 @@ cdw_dll_item_t *cdw_dll_ith_item(cdw_dll_item_t *head, size_t i); /* tested */
 cdw_rv_t cdw_dll_remove_ith_item(cdw_dll_item_t **head, size_t i); /* partially tested */
 size_t cdw_dll_length(cdw_dll_item_t *head); /* tested */
 bool cdw_dll_is_empty(cdw_dll_item_t *head); /* partially tested */
-bool cdw_dll_is_member(cdw_dll_item_t *head, void *data, bool (*pred)(const void *, const void *)); /* tested */
+bool cdw_dll_is_member(cdw_dll_item_t const * head, void const * data, bool (*pred)(void const *, void const *)); /* tested */
 cdw_rv_t cdw_dll_clean(cdw_dll_item_t *head);
 cdw_rv_t cdw_dll_delete(cdw_dll_t **head, cdw_deallocator_t dealloc);
 
