@@ -1,6 +1,6 @@
 /* cdw
  * Copyright (C) 2002 Varkonyi Balazs
- * Copyright (C) 2007 - 2014 Kamil Ignacak
+ * Copyright (C) 2007 - 2016 Kamil Ignacak
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -365,10 +365,10 @@ cdw_rv_t cdw_logging_ask_for_dir_path(void)
 {
 	char *buffer = strdup(cdw_fs_get_tmp_dir_fullpath());
 	/* 2TRANS: this is title of dialog window */
-	cdw_rv_t crv = cdw_fs_ui_file_picker(_("Configuring cdw..."),
-					     _("Enter valid path to directory on non-empty device."),
-					     &(buffer),
-					     CDW_FS_DIR, R_OK | W_OK, CDW_FS_EXISTING);
+	cdw_rv_t crv = cdw_file_picker(_("Configuring cdw..."),
+				       _("Enter valid path to directory on non-empty device."),
+				       &(buffer),
+				       CDW_FS_DIR, R_OK | W_OK, CDW_FS_EXISTING);
 
 	if (crv == CDW_OK) {
 		char *tmp = cdw_string_concat(buffer, "/", cdw_log_file_name, (char *) NULL);
