@@ -3,8 +3,8 @@
 
 #include "main.h"
 
-cdw_rv_t cdw_logging_init(void);
-void cdw_logging_clean(void);
+cdw_rv_t cdw_logging_module_init(void);
+void     cdw_logging_module_clean(void);
 
 void cdw_logging_write(const char *format, ...);
 void cdw_logging_write_separator(void);
