@@ -11,7 +11,7 @@ typedef int (* cdw_regex_handler_t)(regex_t *, regmatch_t *);
 
 typedef struct {
 	const char         *debug_label;   /* used for debug purposes */
-	int                 debug_id;      /* used for debug purposes */
+	cdw_id_t            debug_id;      /* used for debug purposes */
 	const char         *pattern;       /* what we want to capture */
 	regex_t            *regex;         /* compiled pattern */
 	regmatch_t         *matches;       /* stores results of successful match */
@@ -26,7 +26,7 @@ void     cdw_regex_clean_up_regexes_table(cdw_regex_t table[]);
 
 
 /* utility functions used in (all) *_regex.c files */
-void cdw_regex_regcomp_error_handler(const char *caller_name, int errcode, const regex_t *regex, int id);
+void cdw_regex_regcomp_error_handler(const char *caller_name, int errcode, const regex_t *regex, cdw_id_t id);
 int cdw_regex_get_submatch(regmatch_t *matches, unsigned int sub, char *buffer, char *submatch);
 
 
