@@ -10,6 +10,14 @@
 
 void cdw_string_delete(char **string);
 
+/* The argument should have "char *" type. */
+#define CDW_STRING_DELETE(m_string) {			\
+		if ((m_string) != (char *) NULL) {	\
+			free((m_string));		\
+			(m_string) = (char *) NULL;	\
+		}					\
+	}
+
 char *cdw_string_rtrim(char *str);
 char *cdw_string_ltrim(char *str);
 char *cdw_string_trim(const char *string);
@@ -40,7 +48,7 @@ enum {
 
 
 char  *cdw_string_wrap(const char *string, size_t width_max, int align);
-size_t cdw_string_count_lines(const char *string, size_t width_max);
+size_t cdw_string_count_lines_new(const char *string, size_t width_max);
 
 size_t cdw_string_get_line_end(const char *string, size_t chars_max);
 
